@@ -733,7 +733,11 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
 
   /* If a WCS structure has been read, we'll need to pass it to
      `gal_dimension_collapse', so it modifies it respectively. */
-  input->wcs=p->refdata.wcs;
+  if(p->wcs_collapsed==0)
+    {
+      p->wcs_collapsed=1;
+      input->wcs=p->refdata.wcs;
+    }
 
 
   /* Run the relevant library function. */
@@ -770,8 +774,11 @@ arithmetic_collapse(struct arithmeticparams *p, char *token, int operator)
      `collapsed->wcs'. So we'll let the freeing of `input' free the old
      `p->refdata.wcs' structure and we'll put the new one there, then we'll
      set `collapsed->wcs' to `NULL', so the new one isn't freed. */
-  p->refdata.wcs = collapsed->wcs;
-  collapsed->wcs = NULL;
+  if(collapsed->wcs)
+    {
+      p->refdata.wcs = collapsed->wcs;
+      collapsed->wcs = NULL;
+    }
 
 
   /* We'll also need to correct the size of the reference dataset. We'll
@@ -1139,7 +1146,7 @@ reversepolish(struct arithmeticparams *p)
       filename=p->operands->filename;
       if( gal_fits_name_is_fits(filename) )
         {
-          p->operands->data=gal_array_read_one_ch(filename, hdu,
+          p->operands->data=gal_array_read_one_ch(filename, hdu, NULL,
                                                   p->cp.minmapsize);
           p->refdata.wcs=gal_wcs_read(filename, hdu, 0, 0, &p->refdata.nwcs);
           if(!p->cp.quiet) printf(" - %s (hdu %s) is read.\n", filename, hdu);
@@ -1169,7 +1176,11 @@ reversepolish(struct arithmeticparams *p)
       /* Put a copy of the WCS structure from the reference image, it
          will be freed while freeing d1. */
       d1->wcs=p->refdata.wcs;
-      gal_fits_img_write(d1, p->cp.output, NULL, PROGRAM_NAME);
+      if(d1->ndim==1 && p->onedasimage==0)
+        gal_table_write(d1, NULL, p->cp.tableformat, p->cp.output,
+                        "ARITHMETIC", 0);
+      else
+        gal_fits_img_write(d1, p->cp.output, NULL, PROGRAM_NAME);
       if(!p->cp.quiet)
         printf(" - Output written to %s\n", p->cp.output);
     }
