@@ -63,6 +63,8 @@ enum option_keys_enum
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
+  UI_KEY_COLORMAP            = 1000,
+  UI_KEY_RGBTOHSV,
 };
 
 
