@@ -3,9 +3,9 @@ Convolve - Convolve input data with a given kernel.
 Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,6 +32,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/wcs.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/threads.h>
 #include <gnuastro/convolve.h>
 
@@ -58,8 +59,8 @@ complextoreal(double *c, size_t size, int action, double **output)
   double *out, *o, *of;
 
   /* Allocate the space for the real array. */
-  *output=out=gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__,
-                                    "output");
+  *output=out=gal_pointer_allocate(GAL_TYPE_FLOAT64, size, 0, __func__,
+                                   "output");
 
   /* Fill the real array with the derived value from the complex array. */
   of=(o=out)+size;
@@ -207,8 +208,8 @@ frequency_make_padded_complex(struct convolveparams *p)
 
 
   /* Allocate the space for the padded input image and fill it. */
-  pimg=p->pimg=gal_data_malloc_array(GAL_TYPE_FLOAT64, 2*ps0*ps1, __func__,
-                                     "pimg");
+  pimg=p->pimg=gal_pointer_allocate(GAL_TYPE_FLOAT64, 2*ps0*ps1, 0,
+                                    __func__, "pimg");
   for(i=0;i<ps0;++i)
     {
       op=(o=pimg+i*2*ps1)+2*ps1; /* pimg is complex.            */
@@ -222,8 +223,8 @@ frequency_make_padded_complex(struct convolveparams *p)
 
 
   /* Allocate the space for the padded Kernel and fill it. */
-  pker=p->pker=gal_data_malloc_array(GAL_TYPE_FLOAT64, 2*ps0*ps1, __func__,
-                                     "pker");
+  pker=p->pker=gal_pointer_allocate(GAL_TYPE_FLOAT64, 2*ps0*ps1, 0,
+                                    __func__, "pker");
   for(i=0;i<ps0;++i)
     {
       op=(o=pker+i*2*ps1)+2*ps1; /* pker is complex.            */
@@ -760,6 +761,7 @@ void
 convolve(struct convolveparams *p)
 {
   gal_data_t *out, *check;
+  int multidim=p->input->ndim>1;
   struct gal_options_common_params *cp=&p->cp;
 
 
@@ -767,10 +769,10 @@ convolve(struct convolveparams *p)
   if(p->domain==CONVOLVE_DOMAIN_SPATIAL)
     {
       /* Prepare the mesh structure. */
-      gal_tile_full_two_layers(p->input, &cp->tl);
+      if(multidim) gal_tile_full_two_layers(p->input, &cp->tl);
 
       /* Save the tile IDs if they are requested. */
-      if(cp->tl.tilecheckname)
+      if(multidim && cp->tl.tilecheckname)
         {
           check=gal_tile_block_check_tiles(cp->tl.tiles);
           gal_fits_img_write(check, cp->tl.tilecheckname, NULL, PROGRAM_NAME);
@@ -781,12 +783,14 @@ convolve(struct convolveparams *p)
          want to do spatial domain convolution with this Convolve program
          is edge correction. So by default we assume it and will only
          ignore it if the user asks.*/
-      out=gal_convolve_spatial(cp->tl.tiles, p->kernel, cp->numthreads,
-                               !p->noedgecorrection, cp->tl.workoverch);
+      out=gal_convolve_spatial(multidim ? cp->tl.tiles : p->input, p->kernel,
+                               cp->numthreads,
+                               multidim ? !p->noedgecorrection : 1,
+                               multidim ? cp->tl.workoverch : 1 );
 
       /* Clean up: free the actual input and replace it's pointer with the
          convolved dataset to save as output. */
-      gal_tile_full_free_contents(&cp->tl);
+      if(&cp->tl) gal_tile_full_free_contents(&cp->tl);
       gal_data_free(p->input);
       p->input=out;
     }
@@ -794,6 +798,20 @@ convolve(struct convolveparams *p)
     convolve_frequency(p);
 
   /* Save the output (which is in p->input) array. */
-  gal_fits_img_write_to_type(p->input, cp->output, NULL, PROGRAM_NAME,
-                             cp->type);
+  if(p->input->ndim==1)
+    gal_table_write(p->input, NULL, p->cp.tableformat, p->cp.output,
+                    "CONVOLVED", 0);
+  else
+    gal_fits_img_write_to_type(p->input, cp->output, NULL, PROGRAM_NAME,
+                               cp->type);
+
+  /* Write Convolve's parameters as keywords into the first extension of
+     the output. */
+  if( gal_fits_name_is_fits(p->cp.output) )
+    {
+      gal_fits_key_write_filename("input", p->filename, &cp->okeys, 1);
+      gal_fits_key_write_config(&cp->okeys, "Convolve configuration",
+                                "CONVOLVE-CONFIG", cp->output, "0");
+    }
+  printf("  - Output: %s\n", p->cp.output);
 }
