@@ -127,7 +127,23 @@ onecrop_parse_section(struct cropparams *p, size_t *dsize,
           add=1;                /* If it is an asterisk, then add the */
           ++pt;                 /* given value to the maximum size of */
           break;                /* the image. */
+
+        /* Numerical characters signify the start of a number, so we don't
+           need to increment the pointer and can just break out. */
+        case '0': case '1': case '2': case '3': case '4': case '5':
+        case '6': case '7': case '8': case '9': case '-':
+          break;
+
+        /* An un-recognized character should crash the program. */
         default:
+          error(EXIT_FAILURE, 0, "value to `--section' must only contain "
+                "integer numbers and these special characters between them: "
+                "`,', `:', `*' when necessary. But it is `%s' (the first "
+                "non-acceptable character is `%c').\n\n"
+                "Please run the command below to learn more about this "
+                "option in Gnuastro's Crop program:\n\n"
+                "    $ info gnuastro \"Crop section syntax\"\n", p->section,
+                *pt);
           break;
         }
 
@@ -800,12 +816,12 @@ onecrop(struct onecropparams *crp)
       /* A section has been added to the cropped image from this input
          image, so save the information of this image. */
       sprintf(basekeyname, "ICF%zu", crp->numimg);
-      gal_fits_key_write_filename(basekeyname, img->name, &headers);
+      gal_fits_key_write_filename(basekeyname, img->name, &headers, 0);
       sprintf(regionkey, "%sPIX", basekeyname);
       gal_fits_key_list_add_end(&headers, GAL_TYPE_STRING, regionkey,
                                 0, region, 0, "Range of pixels used for "
                                 "this output.", 0, NULL);
-      gal_fits_key_write(ofp, &headers);
+      gal_fits_key_write_in_ptr(&headers, ofp);
 
 
       /* Free the allocated array. */
