@@ -119,6 +119,7 @@ ui_initialize_options(struct matchparams *p,
           break;
         case GAL_OPTIONS_KEY_TYPE:
         case GAL_OPTIONS_KEY_NUMTHREADS:
+        case GAL_OPTIONS_KEY_STDINTIMEOUT:
           cp->coptions[i].flags=OPTION_HIDDEN;
           break;
         }
@@ -169,7 +170,7 @@ parse_opt(int key, char *arg, struct argp_state *state)
         {
           if(p->input2name)
             argp_error(state, "only two arguments (input files) should be "
-                       "given");
+                       "given, not any more");
           else
             p->input2name=arg;
         }
@@ -227,21 +228,34 @@ ui_read_check_only_options(struct matchparams *p)
 static void
 ui_check_options_and_arguments(struct matchparams *p)
 {
-  /* Make sure two input file names were given and if they a FITS file,
-     that a HDU is also given for each. */
+  /* The first input might come from the standard input, in that case, the
+     second input's name is stored in `input1name' and `input2name' will be
+     left to NULL. So we'll first correct for this. */
   if(p->input1name)
     {
-      if( gal_fits_name_is_fits(p->input1name) && p->cp.hdu==NULL )
-        error(EXIT_FAILURE, 0, "no HDU for first input. When the input is "
-              "a FITS file, a HDU must also be specified, you can use the "
-              "`--hdu' (`-h') option and give it the HDU number (starting "
-              "from zero), extension name, or anything acceptable by "
-              "CFITSIO");
+      if(p->input2name==NULL)
+        {
+          p->input2name=p->input1name;
+          p->input1name=NULL;
+        }
     }
   else
-    error(EXIT_FAILURE, 0, "no input file is specified: two inputs are "
-          "necessary");
+    error(EXIT_FAILURE, 0, "no inputs! two inputs are necessary. The first "
+          "can be from the standard input (e.g., a pipe)");
 
+
+  /* If the first input is a FITS file, make sure its HDU is also given. */
+  if( p->input1name
+      && gal_fits_name_is_fits(p->input1name)
+      && p->cp.hdu==NULL )
+    error(EXIT_FAILURE, 0, "no HDU for first input. When the input is "
+          "a FITS file, a HDU must also be specified, you can use the "
+          "`--hdu' (`-h') option and give it the HDU number (starting "
+          "from zero), extension name, or anything acceptable by "
+          "CFITSIO");
+
+  /* If the second input is a FITS file, make sure its HDU is also
+     given. */
   if(p->input2name)
     {
       if( gal_fits_name_is_fits(p->input2name) && p->hdu2==NULL )
@@ -282,7 +296,7 @@ ui_set_mode(struct matchparams *p)
 {
   /* Check if we are in image or catalog mode. We will base the mode on the
      first input, then check with the second. */
-  if( gal_fits_name_is_fits(p->input1name) )
+  if( p->input1name && gal_fits_name_is_fits(p->input1name) )
     p->mode = ( (gal_fits_hdu_format(p->input1name, p->cp.hdu) == IMAGE_HDU)
                 ? MATCH_MODE_WCS
                 : MATCH_MODE_CATALOG );
@@ -403,9 +417,17 @@ ui_read_columns_to_double(struct matchparams *p, char *filename, char *hdu,
     "Please give more specific values to `--ccol1' (column "
     "numberes are the only identifiers guaranteed to be unique).";
 
-  /* Read the columns. */
-  tout=gal_table_read(filename, hdu, cols, cp->searchin, cp->ignorecase,
-                      cp->minmapsize, NULL);
+  /* Read the columns. Note that the first input's name can be NULL (it the
+     user intended to use the standrad input). Also note that this function
+     is called more than one time, so if the Standard input is already read
+     once, we don't want to write a blank list over it (the Standard input
+     will be empty after being read). */
+  if(p->stdinlines==NULL)
+    p->stdinlines=gal_options_check_stdin(filename, p->cp.stdintimeout,
+                                          "input");
+  tout=gal_table_read(filename, hdu, filename ? NULL : p->stdinlines,
+                      cols, cp->searchin, cp->ignorecase, cp->minmapsize,
+                      NULL);
 
   /* A small sanity check. */
   if(gal_list_data_number(tout)!=numcols)
@@ -493,9 +515,8 @@ ui_read_columns(struct matchparams *p)
      names. */
   for(i=0;i<ccol1n;++i)
     {
-      gal_list_str_add(&cols1, strarr1[i], 0);
-      gal_list_str_add(&cols2, strarr2[i], 0);
-      strarr1[i]=strarr2[i]=NULL;  /* So they are not freed later. */
+      gal_list_str_add(&cols1, strarr1[i], 1);
+      gal_list_str_add(&cols2, strarr2[i], 1);
     }
   gal_list_str_reverse(&cols1);
   gal_list_str_reverse(&cols2);
@@ -518,9 +539,6 @@ ui_read_columns(struct matchparams *p)
   /* Free the extra spaces. */
   gal_list_str_free(cols1, 1);
   gal_list_str_free(cols2, 1);
-  gal_data_free(p->ccol1);
-  gal_data_free(p->ccol2);
-  p->ccol1=p->ccol2=NULL;
 }
 
 
@@ -566,35 +584,42 @@ ui_preparations_out_name(struct matchparams *p)
 {
   /* To temporarily keep the original value. */
   uint8_t keepinputdir_orig;
+  char *refname = p->input1name ? p->input1name : p->input2name;
 
   /* Set the output file(s) name(s). */
   if(p->logasoutput)
     {
-      /* Set the logname (as output). */
       if(p->cp.output)
         gal_checkset_allocate_copy(p->cp.output, &p->logname);
       else
         {
           if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
-            p->logname=gal_checkset_automatic_output(&p->cp, p->input1name,
+            p->logname=gal_checkset_automatic_output(&p->cp, refname,
                                                      "_matched.txt");
           else
-            p->logname=gal_checkset_automatic_output(&p->cp, p->input1name,
+            p->logname=gal_checkset_automatic_output(&p->cp, refname,
                                                      "_matched.fits");
         }
 
       /* Make sure a file with this name doesn't exist. */
-      gal_checkset_writable_remove(p->out1name, 0, p->cp.dontdelete);
+      gal_checkset_writable_remove(p->logname, 0, p->cp.dontdelete);
+
+      /* The main output name needs to be available in p->out1name (for the
+         final step when we want to write the input configurations as FITS
+         keywords). */
+      gal_checkset_allocate_copy(p->logname, &p->out1name);
     }
   else
     {
       if(p->outcols)
         {
-          if(p->cp.output==NULL)
-            p->cp.output = gal_checkset_automatic_output(&p->cp,
-                 p->input1name, ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
+          if(p->cp.output)
+            gal_checkset_allocate_copy(p->cp.output, &p->out1name);
+          else
+            p->out1name = gal_checkset_automatic_output(&p->cp,
+                 refname, ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
                                   ? "_matched.txt" : "_matched.fits") );
-          gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
+          gal_checkset_writable_remove(p->out1name, 0, p->cp.dontdelete);
         }
       else
         {
@@ -629,8 +654,7 @@ ui_preparations_out_name(struct matchparams *p)
             {
               if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
                 {
-                  p->out1name=gal_checkset_automatic_output(&p->cp,
-                                                            p->input1name,
+                  p->out1name=gal_checkset_automatic_output(&p->cp, refname,
                                                             "_matched_1.txt");
                   p->out2name=gal_checkset_automatic_output(&p->cp,
                                                             p->input2name,
@@ -638,8 +662,7 @@ ui_preparations_out_name(struct matchparams *p)
                 }
               else
                 {
-                  p->out1name=gal_checkset_automatic_output(&p->cp,
-                                                            p->input1name,
+                  p->out1name=gal_checkset_automatic_output(&p->cp, refname,
                                                             "_matched.fits");
                   gal_checkset_allocate_copy(p->out1name, &p->out2name);
                 }
@@ -756,6 +779,12 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct matchparams *p)
 
   /* Read/allocate all the necessary starting arrays. */
   ui_preparations(p);
+
+
+  /* If the output is a FITS table, prepare all the options as FITS
+     keywords to write in output later. */
+  if(gal_fits_name_is_fits(p->out1name))
+      gal_options_as_fits_keywords(&p->cp);
 }
 
 
@@ -788,6 +817,9 @@ ui_free_report(struct matchparams *p, struct timeval *t1)
   free(p->out1name);
   free(p->out2name);
   free(p->cp.output);
+  gal_data_free(p->ccol1);
+  gal_data_free(p->ccol2);
+  gal_list_str_free(p->stdinlines, 1);
 
   /* Print the final message.
   if(!p->cp.quiet)
