@@ -301,11 +301,11 @@ upperlimit_write_comments(struct mkcatalogparams *p,
       gal_list_str_add(comments, str, 0);
     }
 
-  if( asprintf(&str, "Random number generator name: %s", p->rngname)<0 )
+  if( asprintf(&str, "Random number generator name: %s", p->rng_name)<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(comments, str, 0);
 
-  if( asprintf(&str, "Random number generator seed: %"PRIu64, p->seed)<0 )
+  if( asprintf(&str, "Random number generator seed: %lu", p->rng_seed)<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(comments, str, 0);
 
@@ -390,12 +390,12 @@ upperlimit_write_check(struct mkcatalogparams *p, gal_list_sizet_t *check_x,
 
 
   /* Write exactly what object/clump this table is for. */
-  if( p->checkupperlimit[1]!=GAL_BLANK_INT32 )
-    if( asprintf(&tmp2, ", Clump %d", p->checkupperlimit[1]) <0 )
+  if( p->checkuplim[1]!=GAL_BLANK_INT32 )
+    if( asprintf(&tmp2, ", Clump %d", p->checkuplim[1]) <0 )
       error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   if( asprintf(&tmp, "Upperlimit distribution for Object %d%s",
-               p->checkupperlimit[0],
-               ( p->checkupperlimit[1]==GAL_BLANK_INT32
+               p->checkuplim[0],
+               ( p->checkuplim[1]==GAL_BLANK_INT32
                  ? "" : tmp2) ) <0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(&comments, tmp, 0);
@@ -418,7 +418,7 @@ upperlimit_write_check(struct mkcatalogparams *p, gal_list_sizet_t *check_x,
   y->next=s;
   gal_list_str_reverse(&comments);
   gal_table_write(x, comments, p->cp.tableformat, p->upcheckout,
-                  "UPPERLIMIT_CHECK");
+                  "UPPERLIMIT_CHECK", 0);
 
   /* Inform the user. */
   if(!p->cp.quiet)
@@ -464,7 +464,8 @@ upperlimit_measure(struct mkcatalog_passparams *pp, int32_t clumplab,
             case UI_KEY_UPPERLIMITSIGMA:
             case UI_KEY_UPPERLIMITONESIGMA:
 
-              /* We only need to do this once. */
+              /* We only need to do this once, but the columns can be
+                 requested in any order. */
               if(sigclip==NULL)
                 {
                   /* Calculate the sigma-clipped standard deviation. Since
@@ -474,7 +475,7 @@ upperlimit_measure(struct mkcatalog_passparams *pp, int32_t clumplab,
                   init_size=pp->up_vals->size;
                   sigclip=gal_statistics_sigma_clip(pp->up_vals,
                                                     p->upsigmaclip[0],
-                                                    p->upsigmaclip[1], 1, 1);
+                                                    p->upsigmaclip[1],1,1);
                   pp->up_vals->size=pp->up_vals->dsize[0]=init_size;
                   scarr=sigclip->array;
 
@@ -546,26 +547,26 @@ upperlimit_one_tile(struct mkcatalog_passparams *pp, gal_data_t *tile,
   uint8_t *M=NULL, *st_m=NULL;
   int continueparse, writecheck=0;
   struct gal_list_f32_t *check_s=NULL;
+  size_t d, counter=0, se_inc[2], nfailed=0;
   float *V, *st_v, *uparr=pp->up_vals->array;
-  size_t d, tcounter=0, counter=0, se_inc[2];
   size_t min[2], max[2], increment, num_increment;
   struct gal_list_sizet_t *check_x=NULL, *check_y=NULL;
   int32_t *O, *OO, *oO, *st_o, *st_oo, *st_oc, *oC=NULL;
-  size_t maxcount = p->upnum * MKCATALOG_UPPERLIMIT_STOP_MULTIP;
+  size_t maxfails = p->upnum * MKCATALOG_UPPERLIMIT_MAXFAILS_MULTIP;
   size_t *rcoord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
                                       "rcoord");
 
   /* See if a check table must be created for this distribution. */
-  if( p->checkupperlimit[0]==pp->object )
+  if( p->checkuplim[0]==pp->object )
     {
       /* We are on a clump */
       if( clumplab )
         {
-          if( p->checkupperlimit[1]==clumplab )
+          if( p->checkuplim[1]==clumplab )
             writecheck=1;
         }
       else
-        if( p->checkupperlimit[1]==GAL_BLANK_INT32 )
+        if( p->checkuplim[1]==GAL_BLANK_INT32 )
           writecheck=1;
     }
 
@@ -573,6 +574,7 @@ upperlimit_one_tile(struct mkcatalog_passparams *pp, gal_data_t *tile,
   /* Initializations. */
   tarray=tile->array;
   gsl_rng_set(pp->rng, seed);
+  pp->up_vals->flag &= ~GAL_DATA_FLAG_SORT_CH;
 
 
   /* Set the range of random values for this tile. */
@@ -588,7 +590,7 @@ upperlimit_one_tile(struct mkcatalog_passparams *pp, gal_data_t *tile,
 
 
   /* Continue measuring randomly until we get the desired total number. */
-  while(tcounter<maxcount && counter<p->upnum)
+  while(nfailed<maxfails && counter<p->upnum)
     {
       /* Get the random coordinates. */
       for(d=0;d<ndim;++d)
@@ -655,9 +657,16 @@ upperlimit_one_tile(struct mkcatalog_passparams *pp, gal_data_t *tile,
           else break;
         }
 
+
       /* Further processing is only necessary if this random tile was fully
-         parsed. */
-      if(continueparse) uparr[ counter++ ] = sum;
+         parsed. If it was, we must reset `nfailed' to zero again. */
+      if(continueparse)
+        {
+          nfailed=0;
+          uparr[ counter++ ] = sum;
+        }
+      else ++nfailed;
+
 
       /* If a check is necessary, write in the values. */
       if(writecheck)
@@ -666,10 +675,6 @@ upperlimit_one_tile(struct mkcatalog_passparams *pp, gal_data_t *tile,
           gal_list_sizet_add(&check_y, rcoord[0]+1);
           gal_list_f32_add(&check_s, continueparse ? sum : NAN);
         }
-
-
-      /* Increment the total-counter. */
-      ++tcounter;
     }
 
   /* If a check is necessary, then write the values. */
@@ -718,7 +723,7 @@ upperlimit_calculate(struct mkcatalog_passparams *pp)
   struct mkcatalogparams *p=pp->p;
 
   /* First find the upper limit magnitude for this object. */
-  upperlimit_one_tile(pp, pp->tile, p->seed+pp->object, 0);
+  upperlimit_one_tile(pp, pp->tile, p->rng_seed+pp->object, 0);
 
   /* If a clumps image is present (a clump catalog is requested) and this
      object has clumps, then find the upper limit magnitude for the clumps
@@ -728,13 +733,13 @@ upperlimit_calculate(struct mkcatalog_passparams *pp)
       /* If an upper-limit check image is requested, then make sure that
          the clump label is not more than the number of clumps in this
          object. */
-      if( p->checkupperlimit[0] == pp->object
-          && p->checkupperlimit[1] != GAL_BLANK_INT32
-          && p->checkupperlimit[1] > pp->clumpsinobj )
+      if( p->checkuplim[0] == pp->object
+          && p->checkuplim[1] != GAL_BLANK_INT32
+          && p->checkuplim[1] > pp->clumpsinobj )
         error(EXIT_FAILURE, 0, "object %d has %zu clumps, but an upperlimit "
-              "check table (using the `--checkupperlimit' option) has been "
+              "check table (using the `--checkuplim' option) has been "
               "requested for clump %d", pp->object, pp->clumpsinobj,
-              p->checkupperlimit[1]);
+              p->checkuplim[1]);
 
       /* Make tiles covering the clumps. */
       clumptiles=upperlimit_make_clump_tiles(pp);
@@ -746,7 +751,7 @@ upperlimit_calculate(struct mkcatalog_passparams *pp)
          IDs. */
       for(i=0;i<pp->clumpsinobj;++i)
         {
-          seed = p->seed + p->numobjects + p->numclumps * pp->object + i;
+          seed = p->rng_seed + p->numobjects + p->numclumps * pp->object + i;
           upperlimit_one_tile(pp, &clumptiles[i], seed, i+1);
         }
 
