@@ -131,6 +131,7 @@ ui_initialize_options(struct mknoiseparams *p,
 
         case GAL_OPTIONS_KEY_SEARCHIN:
         case GAL_OPTIONS_KEY_TABLEFORMAT:
+        case GAL_OPTIONS_KEY_STDINTIMEOUT:
           cp->coptions[i].flags=OPTION_HIDDEN;
           break;
         }
@@ -282,7 +283,7 @@ void
 ui_preparations(struct mknoiseparams *p)
 {
   /* Read the input image as a double type */
-  p->input=gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu,
+  p->input=gal_array_read_one_ch_to_type(p->inputname, p->cp.hdu, NULL,
                                      GAL_TYPE_FLOAT64, p->cp.minmapsize);
   p->input->wcs=gal_wcs_read(p->inputname, p->cp.hdu, 0, 0, &p->input->nwcs);
 
@@ -311,13 +312,7 @@ ui_preparations(struct mknoiseparams *p)
 
 
   /* Allocate the random number generator: */
-  gsl_rng_env_setup();
-  p->rng=gsl_rng_alloc(gsl_rng_ranlxs1);
-  p->rng_seed = ( p->envseed
-                  ? gsl_rng_default_seed
-                  : gal_timing_time_based_rng_seed() );
-  gsl_rng_set(p->rng, p->rng_seed);
-  gal_checkset_allocate_copy(gsl_rng_name(p->rng), &p->rng_type);
+  p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
 }
 
 
@@ -385,6 +380,10 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mknoiseparams *p)
   gal_options_print_state(&p->cp);
 
 
+  /* Prepare all the options as FITS keywords to write in output later. */
+  gal_options_as_fits_keywords(&p->cp);
+
+
   /* Check that the options and arguments fit well with each other. Note
      that arguments don't go in a configuration file. So this test should
      be done after (possibly) printing the option values. */
@@ -398,11 +397,11 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mknoiseparams *p)
   /* Everything is ready, notify the user of the program starting. */
   if(!p->cp.quiet)
     {
-      printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
-      sprintf(message, "Random number generator type: %s",
-              gsl_rng_name(p->rng));
+      printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s",
+             ctime(&p->rawtime));
+      sprintf(message, "Random number generator type: %s", p->rng_name);
       gal_timing_report(NULL, message, 1);
-      sprintf(message, "Random number generator seed: %"PRId64, p->rng_seed);
+      sprintf(message, "Random number generator seed: %lu", p->rng_seed);
       gal_timing_report(NULL, message, 1);
     }
 }
@@ -434,7 +433,6 @@ ui_free_report(struct mknoiseparams *p, struct timeval *t1)
 {
   /* Free the allocated arrays: */
   free(p->cp.hdu);
-  free(p->rng_type);
   free(p->cp.output);
   gal_data_free(p->input);
 
