@@ -3,9 +3,9 @@ MakeProfiles - Create mock astronomical profiles.
 MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/git.h>
 #include <gnuastro/fits.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/statistics.h>
 
@@ -41,6 +42,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include "main.h"
 
+#include "ui.h"
 #include "mkprof.h"             /* Needs main.h astrthreads.h */
 #include "oneprofile.h"
 
@@ -126,8 +128,9 @@ saveindividual(struct mkonthread *mkp)
 
   double *crpix;
   long os=p->oversample;
-  size_t i, ndim=p->ndim;
+  gal_fits_list_key_t *keys=NULL;
   struct builtqueue *ibq=mkp->ibq;
+  size_t i, ndim=p->ndim, id=mkp->ibq->id;
   char *filename, *jobname, *outdir=p->outdir;
 
 
@@ -138,12 +141,13 @@ saveindividual(struct mkonthread *mkp)
     filename=p->mergedimgname;
   else
     {
-      asprintf(&filename, "%s%zu_%s", outdir, ibq->id, p->basename);
+      if( asprintf(&filename, "%s%zu_%s", outdir, ibq->id, p->basename)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_checkset_writable_remove(filename, 0, p->cp.dontdelete);
     }
 
 
-  /* Write the array to file (a separately built PSF doesn't need WCS
+  /* Write the array to the file (a separately built PSF doesn't need WCS
      coordinates). */
   if(ibq->ispsf && p->psfinimg==0)
     gal_fits_img_write(ibq->image, filename, NULL, PROGRAM_NAME);
@@ -151,7 +155,8 @@ saveindividual(struct mkonthread *mkp)
     {
       /* Allocate space for the corrected crpix and fill it in. Both
          `crpix' and `fpixel_i' are in FITS order. */
-      crpix=gal_data_malloc_array(GAL_TYPE_FLOAT64, ndim, __func__, "crpix");
+      crpix=gal_pointer_allocate(GAL_TYPE_FLOAT64, ndim, 0, __func__,
+                                 "crpix");
       for(i=0;i<ndim;++i)
         crpix[i] = ((double *)(p->crpix->array))[i] - os*(mkp->fpixel_i[i]-1);
 
@@ -163,10 +168,84 @@ saveindividual(struct mkonthread *mkp)
   ibq->indivcreated=1;
 
 
+  /* Write profile settings into the FITS file. */
+  gal_fits_key_list_add(&keys, GAL_TYPE_STRING, "PROFILE", 0,
+                        ui_profile_name_write(mkp->func), 0,
+                        "Radial function", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT64, "XCENTER", 0,
+                        &p->x[id], 0, "Center of profile in catalog "
+                        "(FITS axis 1)", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT64, "YCENTER", 0,
+                        &p->y[id], 0, "Center of profile in catalog "
+                        "(FITS axis 2)", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "RADIUS", 0,
+                        &p->r[id], 0, "Radial parameter in catalog",
+                        0, NULL);
+  if( mkp->func==PROFILE_SERSIC || mkp->func==PROFILE_MOFFAT )
+    gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PINDEX", 0,
+                          &p->r[id], 0, "Index (Sersic or Moffat) of profile"
+                          "in catalog", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "PA_DEG", 0,
+                        &p->p[id], 0, "Position angle of profile in catalog",
+                        0, "deg");
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "AXISRATIO", 0,
+                        &p->q[id], 0, "Axis ratio of profile in catalog",
+                        0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "MAGNITUDE", 0,
+                        &p->m[id], 0, "Magnitude of profile in catalog",
+                        0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "TRUNCATION", 0,
+                        &p->t[id], 0, "Truncation of profile in catalog",
+                        0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_STRING, "RNGNAME", 0,
+                        (void *)(p->rng_name), 0,
+                        "Name of random number generator", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_ULONG, "RNGSEED", 0,
+                        &mkp->rng_seed, 0, "Seed of random number generator",
+                        0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_SIZE_T, "NUMRANDOM", 0,
+                        &p->numrandom, 0,
+                        "Number of random points in central pixels", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "TOLERANCE", 0,
+                        &p->tolerance, 0,
+                        "Tolerance level to stop random integration",
+                        0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_STRING, "MODE", 0,
+                        p->mode==MKPROF_MODE_IMG?"img":"wcs", 0,
+                        "Coordinates in image or WCS units", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "OVERSAMPLE", 0,
+                        &p->oversample, 0, "Oversampling factor", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "TUNITINP", 0,
+                        &p->tunitinp, 0, "Truncation is in units of pixels, "
+                        "not radius", 0, NULL);
+  if( !isnan(p->zeropoint) )
+      gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "ZEROPOINT", 0,
+                            &p->zeropoint, 0, "Zeropoint magnitude", 0, NULL);
+  if( mkp->func==PROFILE_CIRCUMFERENCE )
+      gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "CIRCUMWIDTH", 0,
+                            &p->circumwidth, 0, "Width of circumference "
+                            "(inward) profiles", 0, NULL);
+  if( mkp->func==PROFILE_FLAT || mkp->func==PROFILE_CIRCUMFERENCE )
+      gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MFORFLATPIX", 0,
+                            &p->mforflatpix, 0, "Magnitude is flat pixel "
+                            "value", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MCOLISBRIGHTNESS", 0,
+                        &p->mcolisbrightness, 0, "Catalog's magnitude is "
+                        "actually brightness", 0, NULL);
+  gal_fits_key_list_add(&keys, GAL_TYPE_UINT8, "MAGATPEAK", 0,
+                        &p->magatpeak, 0, "Magnitude is for peak pixel, "
+                        "not full profile", 0, NULL);
+
+  gal_fits_key_list_reverse(&keys);
+  gal_fits_key_write_config(&keys, "Profile configuration", "PROFILE-CONFIG",
+                            filename, "0");
+
+
   /* Report if in verbose mode. */
   if(!p->cp.quiet)
     {
-      asprintf(&jobname, "%s created.", filename);
+      if( asprintf(&jobname, "%s created.", filename)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(NULL, jobname, 2);
       free(jobname);
     }
@@ -217,8 +296,13 @@ mkprof_build_single(struct mkonthread *mkp, long *fpixel_i, long *lpixel_i,
 
   /* Set the seed of the random number generator if the
      environment is not to be used. */
-  if(mkp->p->envseed==0)
-    gsl_rng_set(mkp->rng, gal_timing_time_based_rng_seed());
+  if(mkp->p->envseed)
+    mkp->rng_seed=mkp->p->rng_seed;
+  else
+    {
+      mkp->rng_seed=gal_timing_time_based_rng_seed();
+      gsl_rng_set(mkp->rng, mkp->rng_seed);
+    }
 
   /* Make the profile */
   oneprofile_make(mkp);
@@ -257,8 +341,7 @@ mkprof_build_single(struct mkonthread *mkp, long *fpixel_i, long *lpixel_i,
           /* If a crop is needed, set the starting pointer. */
           ind=gal_dimension_coord_to_index(ndim, ibq->image->dsize,
                                            start_indiv);
-          ptr=gal_data_ptr_increment(ibq->image->array, ind,
-                                     ibq->image->type);
+          ptr=gal_pointer_increment(ibq->image->array, ind, ibq->image->type);
         }
       else ptr=ibq->image->array;
       ibq->overlap_i=gal_data_alloc(ptr, ibq->image->type, ndim, dsize, NULL,
@@ -268,7 +351,7 @@ mkprof_build_single(struct mkonthread *mkp, long *fpixel_i, long *lpixel_i,
 
       /* Define the merged overlap tile. */
       ind=gal_dimension_coord_to_index(ndim, p->out->dsize, start_mrg);
-      ptr=gal_data_ptr_increment(p->out->array, ind, p->out->type);
+      ptr=gal_pointer_increment(p->out->array, ind, p->out->type);
       ibq->overlap_m=gal_data_alloc(ptr, p->out->type, ndim, dsize, NULL,
                                     0, -1, NULL, NULL, NULL);
       ibq->overlap_m->block=p->out;
@@ -537,8 +620,9 @@ mkprof_write(struct mkprofparams *p)
       ++complete;
       if(!p->cp.quiet && p->num>1)
         {
-          asprintf(&jobname, "row %zu complete, %zu left to go",
-                   ibq->id+1, num-complete);
+          if( asprintf(&jobname, "row %zu complete, %zu left to go",
+                       ibq->id+1, num-complete)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           gal_timing_report(NULL, jobname, 2);
           free(jobname);
         }
@@ -574,10 +658,16 @@ mkprof_write(struct mkprofparams *p)
       /* Clean up */
       gal_data_free(out);
 
+      /* Write the configuration keywords. */
+      gal_fits_key_write_filename("input", p->catname, &p->cp.okeys, 1);
+      gal_fits_key_write_config(&p->cp.okeys, "MakeProfiles configuration",
+                                "MKPROF-CONFIG", p->mergedimgname, "0");
+
       /* In verbose mode, print the information. */
       if(!p->cp.quiet)
         {
-          asprintf(&jobname, "%s created.", p->mergedimgname);
+          if( asprintf(&jobname, "%s created.", p->mergedimgname)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
           gal_timing_report(&t1, jobname, 1);
           free(jobname);
         }
@@ -641,7 +731,7 @@ mkprof(struct mkprofparams *p)
      ignore it. */
   if(p->out)
     {
-      onaxes=gal_data_malloc_array(GAL_TYPE_LONG, ndim, __func__, "onaxes");
+      onaxes=gal_pointer_allocate(GAL_TYPE_LONG, ndim, 0, __func__, "onaxes");
       for(fi=0; fi < ndim; ++fi)
         {
           i=ndim-fi-1;
@@ -701,7 +791,8 @@ mkprof(struct mkprofparams *p)
   /* Write the log file. */
   if(p->cp.log)
     {
-      asprintf(&tmp, "Zeropoint: %g", p->zeropoint);
+      if( asprintf(&tmp, "Zeropoint: %g", p->zeropoint)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_list_str_add(&comments, tmp, 0);
       gal_checkset_writable_remove(LOGFILENAME, 0, p->cp.dontdelete);
       gal_table_write_log(p->log, PROGRAM_STRING, &p->rawtime, comments,
