@@ -3,9 +3,9 @@ MakeProfiles - Create mock astronomical profiles.
 MakeProfiles is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -34,6 +34,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gsl/gsl_integration.h> /* gsl_integration_qng  */
 
 #include <gnuastro/fits.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/statistics.h>
 
@@ -172,7 +173,9 @@ oneprofile_r_circle(size_t index, struct mkonthread *mkp)
 float
 oneprofile_randompoints(struct mkonthread *mkp)
 {
+  double r_before=mkp->r;
   double range[2], sum=0.0f;
+  double coord_before[2]={mkp->coord[0], mkp->coord[1]};
   size_t i, j, numrandom=mkp->p->numrandom, ndim=mkp->p->ndim;
 
   /* Set the range in each dimension. */
@@ -188,6 +191,15 @@ oneprofile_randompoints(struct mkonthread *mkp)
       sum+=mkp->profile(mkp);
     }
 
+  /* Reset the original distance and coordinate of the pixel and return the
+     average random value. The resetting is mostly redundant (only useful
+     in checks), but since it has a very negligible cost (compared to the
+     random checks above) cost, its good to reset it to help in debugging
+     when necessary (avoid confusion when un-commenting the checks in
+     `oneprofile_pix_by_pix'). */
+  mkp->r=r_before;
+  mkp->coord[0]=coord_before[0];
+  mkp->coord[1]=coord_before[1];
   return sum/numrandom;
 }
 
@@ -346,9 +358,9 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
 
   /* Allocate the `byt' array. It is used as a flag to make sure that we
      don't re-calculate the profile value on a pixel more than once. */
-  byt = gal_data_calloc_array(GAL_TYPE_UINT8,
-                              gal_dimension_total_size(ndim, dsize),
-                              __func__, "byt");
+  byt = gal_pointer_allocate(GAL_TYPE_UINT8,
+                             gal_dimension_total_size(ndim, dsize), 1,
+                             __func__, "byt");
 
   /* Start the queue: */
   byt[p]=1;
@@ -376,7 +388,7 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
           oneprofile_r_el(mkp);
           if(mkp->r > truncr) continue;
 
-          /* Find the value for this pixel: */
+          /* Set the range for this pixel. */
           for(i=0;i<ndim;++i)
             {
               mkp->lower[i]  = mkp->coord[i] - hp;
@@ -389,6 +401,12 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
           if (fabs(array[p]-approx)/array[p] < tolerance)
             use_rand_points=0;
 
+          /* For a check:
+          printf("coord: %g, %g\n", mkp->coord[0], mkp->coord[1]);
+          printf("r_rand: %g (rand: %g, center: %g)\n\n", mkp->r, array[p],
+                 approx);
+          */
+
           /* Save the peak flux if this is the first pixel: */
           if(ispeak) { mkp->peakflux=array[p]; ispeak=0; }
 
@@ -440,6 +458,10 @@ oneprofile_pix_by_pix(struct mkonthread *mkp)
       /* Find the value for this pixel: */
       array[p]=profile(mkp);
 
+      /* For a check:
+      printf("r_center: %g\n", mkp->r);
+      */
+
       /* Save the peak flux if this is the first pixel: */
       if(ispeak) { mkp->peakflux=array[p]; ispeak=0; }
 
@@ -617,10 +639,12 @@ oneprof_set_prof_params(struct mkonthread *mkp)
       mkp->correction       = 0;
       break;
 
+
+
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us so we can correct "
-            "this problem. The profile code %u is not recognized.", __func__,
-            mkp->func);
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us so we can "
+            "correct this problem. The profile code %u is not recognized.",
+            __func__, mkp->func);
     }
 }
 
