@@ -144,7 +144,7 @@ ui_profile_name_read(char *string, size_t row)
 
 
 
-static char *
+char *
 ui_profile_name_write(int profile_code)
 {
   switch(profile_code)
@@ -487,13 +487,13 @@ ui_read_check_only_options(struct mkprofparams *p)
           "a magnitude, not brightness).");
 
 
-  /* Make sure no zero value is given for the `--naxis' option (only when
-     it is necessary). */
+  /* Make sure no zero value is given for the `--mergedsize' option (only
+     when it is necessary). */
   if(p->dsize && p->backname==NULL)
     for(i=0;p->dsize[i]!=GAL_BLANK_SIZE_T;++i)
       if(p->dsize[i]==0)
-        error(EXIT_FAILURE, 0, "values to `--naxes' option must not be "
-              "zero");
+        error(EXIT_FAILURE, 0, "values to `--mergedsize' option must not "
+              "be zero");
 }
 
 
@@ -529,10 +529,6 @@ ui_check_options_and_arguments(struct mkprofparams *p)
             error(EXIT_FAILURE, 0, "no `hdu' specified for the input FITS "
                   "table '%s', to ", p->catname);
         }
-      else
-        error(EXIT_FAILURE, 0, "no input catalog provided. To build "
-              "profiles, you need to give a catalog/table containing "
-              "the information of the profiles");
     }
 
 
@@ -554,7 +550,10 @@ ui_check_options_and_arguments(struct mkprofparams *p)
     {
       gal_checkset_allocate_copy(p->cp.output, &p->outdir);
       gal_checkset_check_dir_write_add_slash(&p->outdir);
-      tmpname=gal_checkset_automatic_output(&p->cp, p->catname, ".fits");
+      tmpname=gal_checkset_automatic_output(&p->cp,
+                                            ( p->catname
+                                              ? p->catname
+                                              : "makeprofiles" ), ".fits");
       p->mergedimgname=gal_checkset_malloc_cat(p->outdir, tmpname);
       free(tmpname);
     }
@@ -596,8 +595,8 @@ ui_read_cols(struct mkprofparams *p)
   int checkblank;
   size_t i, counter=0;
   char *colname=NULL, **strarr;
-  gal_list_str_t *colstrs=NULL, *ccol;
   gal_data_t *cols, *tmp, *corrtype=NULL;
+  gal_list_str_t *ccol, *lines, *colstrs=NULL;
 
   /* The coordinate columns are a linked list of strings. */
   ccol=p->ccol;
@@ -621,8 +620,17 @@ ui_read_cols(struct mkprofparams *p)
   gal_list_str_reverse(&colstrs);
 
   /* Read the desired columns from the file. */
-  cols=gal_table_read(p->catname, p->cp.hdu, colstrs, p->cp.searchin,
+  lines=gal_options_check_stdin(p->catname, p->cp.stdintimeout, "input");
+  cols=gal_table_read(p->catname, p->cp.hdu, lines, colstrs, p->cp.searchin,
                       p->cp.ignorecase, p->cp.minmapsize, NULL);
+  gal_list_str_free(lines, 1);
+
+  /* The name of the input catalog is only for informative steps from now
+     on (we won't be dealing with the actual file any more). So if the
+     standard input was used (therefore `catname==NULL', set it to
+     `stdin'). */
+  if(p->catname==NULL)
+    gal_checkset_allocate_copy("standard-input", &p->catname);
 
   /* Set the number of objects. */
   p->num=cols->size;
@@ -696,10 +704,11 @@ ui_read_cols(struct mkprofparams *p)
 
           /* Check if there is no negative or zero-radius profile. */
           for(i=0;i<p->num;++i)
-            if(p->r[i]<=0.0f)
+            if(p->f[i]!=PROFILE_POINT && p->r[i]<=0.0f)
               error(EXIT_FAILURE, 0, "%s: row %zu, the radius value %g is "
-                    "not acceptable. It has to be larger than 0", p->catname,
-                    i+1, p->r[i]);
+                    "not acceptable for a `%s' profile. It has to be larger "
+                    "than 0", p->catname, i+1, p->r[i],
+                    ui_profile_name_write(p->f[i]));
           break;
 
 
@@ -724,10 +733,11 @@ ui_read_cols(struct mkprofparams *p)
 
           /* Check if there is no negative or >1.0f axis ratio. */
           for(i=0;i<p->num;++i)
-            if(p->q[i]<=0.0f || p->q[i]>1.0f)
+            if( p->f[i]!=PROFILE_POINT && (p->q[i]<=0.0f || p->q[i]>1.0f) )
               error(EXIT_FAILURE, 0, "%s: row %zu, the axis ratio value %g "
-                    "is not acceptable. It has to be >0 and <=1", p->catname,
-                    i+1, p->q[i]);
+                    "is not acceptable for a `%s' profile. It has to be >0 "
+                    "and <=1", p->catname, i+1, p->q[i],
+                    ui_profile_name_write(p->f[i]));
           break;
 
 
@@ -746,10 +756,11 @@ ui_read_cols(struct mkprofparams *p)
 
           /* Check if there is no negative or zero truncation radius. */
           for(i=0;i<p->num;++i)
-            if(p->t[i]<=0.0f)
+            if(p->f[i]!=PROFILE_POINT && p->t[i]<=0.0f)
               error(EXIT_FAILURE, 0, "%s: row %zu, the truncation radius "
-                    "value %g is not acceptable. It has to be larger than 0",
-                    p->catname, i+1, p->t[i]);
+                    "value %g is not acceptable for a `%s' profile. It has "
+                    "to be larger than 0", p->catname, i+1, p->t[i],
+                    ui_profile_name_write(p->f[i]));
           break;
 
 
@@ -993,7 +1004,6 @@ ui_prepare_canvas(struct mkprofparams *p)
 {
   float *f, *ff;
   double truncr;
-  gal_data_t *keysll;
   long width[2]={1,1};
   int status=0, setshift=0;
   size_t i, nshift=0, *dsize=NULL, ndim_counter;
@@ -1002,52 +1012,13 @@ ui_prepare_canvas(struct mkprofparams *p)
      image to build the profiles over. */
   if(p->backname)
     {
-      /* Make sure the kernel option is not called. */
-      if(p->kernel)
-        error(EXIT_FAILURE, 0, "the `--kernel' and `--background' options "
-              "cannot be called together");
-
-      /* Small sanity check. */
-      if(p->backhdu==NULL)
-        error(EXIT_FAILURE, 0, "no hdu specified for the background image "
-              "%s. Please run again `--backhdu' option", p->backname);
-
       /* Read in the background image and its coordinates, note that when
          no merged image is desired, we just need the WCS information of
          the background image and the number of its dimensions. So
          `ndim==0' and what `dsize' points to is irrelevant. */
-      if(p->nomerged)
+      p->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0, &p->nwcs);
+      if(p->nomerged==0)
         {
-          /* Get the number of the background image's dimensions. */
-          keysll=gal_data_array_calloc(1);
-          keysll->name="NAXIS";   keysll->type=GAL_TYPE_SIZE_T;
-          gal_fits_key_read(p->backname, p->backhdu, keysll, 0, 0);
-          p->ndim = *(size_t *)(keysll->array);
-          keysll->name=NULL;
-          gal_data_array_free(keysll, 1, 1);
-
-          /* Read the WCS structure of the background image. */
-          p->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0, &p->nwcs);
-        }
-      else
-        {
-          /* Read the image. */
-          p->out=gal_array_read_one_ch_to_type(p->backname, p->backhdu,
-                                               GAL_TYPE_FLOAT32,
-                                               p->cp.minmapsize);
-          p->out->wcs=gal_wcs_read(p->backname, p->backhdu, 0, 0,
-                                   &p->out->nwcs);
-
-          /* Put the WCS structure and number of dimensions in the
-             MakeProfiles's main structure for generality. The WCS
-             structure will be put back in the end when writing. */
-          p->wcs=p->out->wcs;
-          p->nwcs=p->out->nwcs;
-          p->ndim=p->out->ndim;
-          p->out->wcs=NULL;
-          p->out->nwcs=0;
-
-
           /* If p->dsize was given as an option, free it. */
           if( p->dsize ) free(p->dsize);
 
@@ -1061,14 +1032,6 @@ ui_prepare_canvas(struct mkprofparams *p)
             {ff=(f=p->out->array)+p->out->size; do *f++=0.0f; while(f<ff);}
         }
 
-
-      /* Currently, things are only implemented for 2D. */
-      if(p->ndim!=2)
-        error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions. Currently "
-              "only a 2 dimensional background image is acceptable",
-              p->backname, p->backhdu, p->ndim);
-
-
       /* When a background image is specified, oversample must be 1 and
          there is no shifts. */
       p->oversample=1;
@@ -1078,16 +1041,6 @@ ui_prepare_canvas(struct mkprofparams *p)
     }
   else
     {
-      /* Get the number of dimensions. */
-      ndim_counter=0;
-      for(i=0;p->dsize[i]!=GAL_BLANK_SIZE_T;++i) ++ndim_counter;
-      p->ndim=ndim_counter;
-
-      /* Currently, things are only implemented for 2D. */
-      if(p->ndim!=2)
-        error(EXIT_FAILURE, 0, "%zu numbers given to `--naxis', only 2 "
-              "values may be given", p->ndim);
-
       /* If any of the shift elements are zero, the others should be too!*/
       if(p->shift && p->shift[0] && p->shift[1])
         {
@@ -1333,9 +1286,91 @@ ui_make_log(struct mkprofparams *p)
 
 
 
+static void
+ui_read_ndim(struct mkprofparams *p)
+{
+  gal_data_t *keysll;
+  size_t i, ndim_counter;
+
+  if(p->kernel)
+    {
+      /* The kernel's dimensionality is fixed. */
+      p->ndim=2;
+
+      /* Make sure the kernel and background are not given together. */
+      if(p->backname)
+        error(EXIT_FAILURE, 0, "the `--kernel' and `--background' options "
+              "cannot be called together");
+    }
+  else
+    {
+      /* Packground image is given. */
+      if(p->backname)
+        {
+          /* Small sanity check. */
+          if(p->backhdu==NULL)
+            error(EXIT_FAILURE, 0, "no hdu specified for the background "
+                  "image %s. Please run again `--backhdu' option",
+                  p->backname);
+
+          /* If `--nomerged' is given, we don't actually need to load the
+             image, we just need its WCS later. */
+          if(p->nomerged)
+            {
+              /* Get the number of the background image's dimensions. */
+              keysll=gal_data_array_calloc(1);
+              keysll->name="NAXIS";   keysll->type=GAL_TYPE_SIZE_T;
+              gal_fits_key_read(p->backname, p->backhdu, keysll, 0, 0);
+              p->ndim = *(size_t *)(keysll->array);
+              keysll->name=NULL;
+              gal_data_array_free(keysll, 1, 1);
+            }
+          else
+            {
+              /* Read the image. */
+              p->out=gal_array_read_one_ch_to_type(p->backname, p->backhdu,
+                                                   NULL, GAL_TYPE_FLOAT32,
+                                                   p->cp.minmapsize);
+              p->ndim=p->out->ndim;
+            }
+
+          /* Make sure the dimensionality is supported. */
+          if(p->ndim!=2)
+            error(EXIT_FAILURE, 0, "%s (hdu %s) has %zu dimensions. Currently "
+                  "only 2 dimensional outputs can be produced",
+                  p->backname, p->backhdu, p->ndim);
+        }
+      else
+        {
+          /* Get the number of dimensions from the user's options. */
+          ndim_counter=0;
+          for(i=0;p->dsize[i]!=GAL_BLANK_SIZE_T;++i) ++ndim_counter;
+          p->ndim=ndim_counter;
+
+          /* Make sure the dimensionality is supported. */
+          if(p->ndim!=2)
+            error(EXIT_FAILURE, 0, "%zu values given to `--mergedsize'. "
+                  "Currently only 2 dimensional outputs can be produced",
+                  p->ndim);
+        }
+    }
+}
+
+
+
+
+
 static void
 ui_preparations(struct mkprofparams *p)
 {
+  /* Set the output dimensionality (necessary to know which columns to
+     use). */
+  ui_read_ndim(p);
+
+  /* Read in all the columns (necessary for `--prepforconf' when we want to
+     build the profiles). */
+  ui_prepare_columns(p);
+
   /* If the kernel option was given, some parameters need to be
      over-written: */
   if(p->kernel)
@@ -1353,16 +1388,12 @@ ui_preparations(struct mkprofparams *p)
   else
     ui_prepare_canvas(p);
 
-  /* Read in all the columns. */
-  ui_prepare_columns(p);
-
   /* Read the (possible) RA/Dec inputs into X and Y for the builder.*/
   if(p->wcs)
     ui_finalize_coordinates(p);
 
-  /* Allocate the random number generator: */
-  gsl_rng_env_setup();
-  p->rng=gsl_rng_alloc(gsl_rng_ranlxs1);
+  /* Prepare the random number generator. */
+  p->rng=gal_checkset_gsl_rng(p->envseed, &p->rng_name, &p->rng_seed);
 
   /* Make the log linked list. */
   ui_make_log(p);
@@ -1397,7 +1428,7 @@ ui_print_intro(struct mkprofparams *p)
 
   if(p->cp.quiet) return;
 
-  printf(PROGRAM_NAME" started on %s", ctime(&p->rawtime));
+  printf(PROGRAM_NAME" "PACKAGE_VERSION" started on %s", ctime(&p->rawtime));
 
   if(p->kernel)
     {
@@ -1437,14 +1468,11 @@ ui_print_intro(struct mkprofparams *p)
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_timing_report(NULL, jobname, 1);
   free(jobname);
-  if(p->envseed)
-    {
-      if( asprintf(&jobname, "RNG seed for all profiles: %lu",
-                   gsl_rng_default_seed)<0 )
-        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-      gal_timing_report(NULL, jobname, 1);
-      free(jobname);
-    }
+
+  if( asprintf(&jobname, "Basic RNG seed: %lu", p->rng_seed)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+  gal_timing_report(NULL, jobname, 1);
+  free(jobname);
 
   if(p->kernel==NULL)
     {
@@ -1501,6 +1529,10 @@ ui_read_check_inputs_setup(int argc, char *argv[], struct mkprofparams *p)
   gal_options_print_state(&p->cp);
 
 
+  /* Prepare all the options as FITS keywords to write in output later. */
+  gal_options_as_fits_keywords(&p->cp);
+
+
   /* Check that the options and arguments fit well with each other. Note
      that arguments don't go in a configuration file. So this test should
      be done after (possibly) printing the option values. */
