@@ -1,11 +1,11 @@
 /*********************************************************************
-NoiseChisel - Detect and segment signal in a noisy dataset.
+NoiseChisel - Detect signal in a noisy dataset.
 NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -29,18 +29,21 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 
 #include <gnuastro/fits.h>
+#include <gnuastro/label.h>
+#include <gnuastro/blank.h>
 #include <gnuastro/binary.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/statistics.h>
 
 #include <gnuastro-internal/timing.h>
+#include <gnuastro-internal/checkset.h>
 
 #include "main.h"
 
 #include "ui.h"
 #include "sky.h"
-#include "clumps.h"
 #include "threshold.h"
 
 
@@ -53,8 +56,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 void
 detection_initial(struct noisechiselparams *p)
 {
+  float *f;
   char *msg;
   uint8_t *b, *bf;
+  int resetblank=0;
   struct timeval t0, t1;
 
 
@@ -76,13 +81,23 @@ detection_initial(struct noisechiselparams *p)
     }
 
 
+  /* Remove any blank elements from the binary image if requested. */
+  if(p->blankasforeground==0 && gal_blank_present(p->binary,0))
+    {
+      resetblank=1;
+      bf=(b=p->binary->array)+p->binary->size;
+      do *b = *b==GAL_BLANK_UINT8 ? 0 : *b; while(++b<bf);
+    }
+
+
   /* Erode the image. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
   gal_binary_erode(p->binary, p->erode, p->erodengb==4 ? 1 : 2, 1);
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "Eroded %zu time%s (%zu-connectivity).", p->erode,
-               p->erode>1?"s":"", p->erodengb);
+      if( asprintf(&msg, "Eroded %zu time%s (%zu-connected).", p->erode,
+                   p->erode>1?"s":"", p->erodengb)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
@@ -104,13 +119,23 @@ detection_initial(struct noisechiselparams *p)
   gal_binary_open(p->binary, p->opening, p->openingngb==4 ? 1 : 2, 1);
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "Opened (depth: %zu, %s connectivity).",
-              p->opening, p->openingngb==4 ? "4" : "8");
+      if( asprintf(&msg, "Opened (depth: %zu, %zu-connected).",
+                   p->opening, p->openingngb)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
 
 
+  /* Reset the blank values (if requested). */
+  if(resetblank)
+    {
+      f=p->input->array;
+      bf=(b=p->binary->array)+p->binary->size;
+      do *b = isnan(*f++) ? GAL_BLANK_UINT8 : *b; while(++b<bf);
+    }
+
+
   /* Label the connected components. */
   p->numinitialdets=gal_binary_connected_components(p->binary, &p->olabel,
                                                     p->binary->ndim);
@@ -125,7 +150,9 @@ detection_initial(struct noisechiselparams *p)
   /* Report the ending of initial detection. */
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "%zu initial detections found.", p->numinitialdets);
+      if( asprintf(&msg, "%zu initial detections found.",
+                   p->numinitialdets)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t0, msg, 1);
       free(msg);
     }
@@ -212,6 +239,7 @@ detection_fill_holes_open(void *in_prm)
   struct noisechiselparams *p=fho_prm->p;
 
   void *tarray;
+  uint8_t *b, *bf;
   gal_data_t *tile, *copy, *tblock;
   size_t i, dsize[]={1,1,1,1,1,1,1,1,1,1}; /* For upto 10-Dimensions! */
 
@@ -239,12 +267,23 @@ detection_fill_holes_open(void *in_prm)
       /* Copy the tile into the contiguous patch of memory to work on, but
          first reset the size element so `gal_data_copy_to_allocated' knows
          there is enough space. */
+      copy->flag=0;
       copy->size=p->maxltcontig;
       gal_data_copy_to_allocated(tile, copy);
 
+      /* Take blank values to the background (set them to zero) if
+         necsesary. */
+      if( p->blankasforeground==0
+          && gal_blank_present(p->input,0)
+          && gal_blank_present(copy, 1) )
+        {
+          bf=(b=copy->array)+copy->size;
+          do *b = *b==GAL_BLANK_UINT8 ? 0 : *b; while(++b<bf);
+        }
+
       /* Fill the holes in this tile: holes with maximal connectivity means
          that they are most strongly bounded. */
-      gal_binary_fill_holes(copy, copy->ndim, -1);
+      gal_binary_holes_fill(copy, p->holengb==4 ? 1 : 2, -1);
       if(fho_prm->step==1)
         {
           detection_write_in_large(tile, copy);
@@ -285,6 +324,8 @@ static size_t
 detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
                       gal_data_t *worklab, int s0d1)
 {
+  float *f;
+  uint8_t *b, *bf;
   gal_data_t *bin;
   struct fho_params fho_prm={0, NULL, workbin, worklab, p};
 
@@ -304,9 +345,9 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
      thread. `maxltcontig' is the maximum contiguous patch of memory needed
      to store all tiles. Finally, since we are working on a `uint8_t' type,
      the size of each element is only 1 byte. */
-  fho_prm.copyspace=gal_data_malloc_array(GAL_TYPE_UINT8,
-                                          p->cp.numthreads*p->maxltcontig,
-                                          __func__, "fho_prm.copyspace");
+  fho_prm.copyspace=gal_pointer_allocate(GAL_TYPE_UINT8,
+                                         p->cp.numthreads*p->maxltcontig, 0,
+                                         __func__, "fho_prm.copyspace");
 
 
   /* Fill the holes and open on each large tile. When no check image is
@@ -333,6 +374,14 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
           gal_threads_spin_off(detection_fill_holes_open, &fho_prm,
                                p->ltl.tottiles, p->cp.numthreads);
 
+          /* Reset the blank values (if they were changed). */
+          if( p->blankasforeground==0 && gal_blank_present(p->input,0) )
+            {
+              f=p->input->array;
+              bf=(b=workbin->array)+workbin->size;
+              do *b = isnan(*f++) ? GAL_BLANK_UINT8 : *b; while(++b<bf);
+            }
+
           /* Set the extension name based on the step. */
           switch(fho_prm.step)
             {
@@ -404,13 +453,14 @@ static void
 detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
                            gal_data_t *snind, int s0d1D2)
 {
-  char *str;
+  char *str, *extname=NULL;
   gal_list_str_t *comments=NULL;
 
   /* Comment for extension on further explanation. */
-  asprintf(&str, "See also: `%s' HDU of output with "
-           "`--checkdetection'", ( s0d1D2<2
-                                   ? "PSEUDOS-FOR-SN": "DILATED" ));
+  if( asprintf(&str, "See also: `%s' HDU of output with "
+               "`--checkdetection'", ( s0d1D2<2
+                                       ? "PSEUDOS-FOR-SN": "DILATED" ))<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   gal_list_str_add(&comments, str, 0);
 
 
@@ -423,11 +473,15 @@ detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
   gal_list_str_add(&comments, str, 1);
 
 
-  /* Set the file name. */
+  /* Set the file name and write the table. */
   str = ( s0d1D2
           ? ( s0d1D2==2 ? p->detsn_D_name : p->detsn_d_name )
           : p->detsn_s_name );
-  threshold_write_sn_table(p, sn, snind, str, comments);
+  if( p->cp.tableformat!=GAL_TABLE_FORMAT_TXT )
+    extname = ( s0d1D2
+                ? ( s0d1D2==2 ? "GROWN_DETECTION_SN" : "DET_PSEUDODET_SN" )
+                : "SKY_PSEUDODET_SN" );
+  threshold_write_sn_table(p, sn, snind, str, comments, extname);
   gal_list_str_free(comments, 1);
 
 
@@ -453,14 +507,13 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   gal_data_t *sn, *snind;
   int32_t *plabend, *indarr=NULL;
   double ave, err, *xy, *brightness;
+  float s, ss, *f, *ff, *fs, *sky=p->sky->array;
   size_t ind, ndim=p->input->ndim, xyncols=1+ndim;
   size_t i, *area, counter=0, *dsize=p->input->dsize;
-  float *img=p->input->array, *f=p->input->array, *ff=f+p->input->size;
   int32_t *plab = worklab->array, *dlab = s0d1D2 ? NULL : p->olabel->array;
-  size_t *coord=gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim, __func__,
+  size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
                                       "coord");
 
-
   /* Sanity check. */
   if(p->input->type!=GAL_TYPE_FLOAT32)
     error(EXIT_FAILURE, 0, "%s: the input dataset must be float32 type, "
@@ -477,25 +530,27 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   /* Allocate all the necessary arrays, note that since we want to put each
      object's information into the same index, the number of allocated
      spaces has to be `tablen=num+1'. */
-  area       = gal_data_calloc_array(GAL_TYPE_SIZE_T,  tablen, __func__,
-                                     "area");
-  brightness = gal_data_calloc_array(GAL_TYPE_FLOAT64, tablen, __func__,
+  area       = gal_pointer_allocate(GAL_TYPE_SIZE_T,  tablen, 1, __func__,
+                                    "area");
+  brightness = gal_pointer_allocate(GAL_TYPE_FLOAT64, tablen, 1, __func__,
                                      "brightness");
-  xy         = gal_data_calloc_array(GAL_TYPE_FLOAT64, xyncols*tablen,
+  xy         = gal_pointer_allocate(GAL_TYPE_FLOAT64, xyncols*tablen, 1,
                                      __func__, "xy");
   flag       = ( s0d1D2==0
-                 ? gal_data_calloc_array(GAL_TYPE_UINT8, tablen, __func__,
+                 ? gal_pointer_allocate(GAL_TYPE_UINT8, tablen, 1, __func__,
                                          "flag")
                  : NULL );
   sn         = gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &tablen, NULL, 1,
                               p->cp.minmapsize, "SIGNAL-TO-NOISE", "ratio",
                               NULL);
-  snind      = ( p->checkdetsn==0 ? NULL
+  snind      = ( p->checksn==0 ? NULL
                  : gal_data_alloc(NULL, GAL_TYPE_INT32, 1, &tablen, NULL, 1,
                                   p->cp.minmapsize, "LABEL", "counter",
                                   NULL) );
 
   /* Go over all the pixels and get the necessary information. */
+  fs = f = p->input->array;
+  ff = f + p->input->size;
   do
     {
       /* All this work is only necessary when we are actually on a
@@ -518,12 +573,15 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
 
           /* Save all the necessary values. */
           ++area[*plab];
-          brightness[*plab] += *f;
-          if( *f > 0.0f )  /* For calculatiing the approximate center, */
+          gal_dimension_index_to_coord(f-fs, ndim, dsize, coord);
+          s  = sky[ gal_tile_full_id_from_coord(&p->cp.tl, coord) ];
+          ss = *f-s;
+          brightness[*plab] += ss;
+          if( ss > 0.0f )  /* For calculatiing the approximate center, */
             {              /* necessary for calculating Sky and STD.   */
-              xy[*plab*xyncols  ] += *f;
-              xy[*plab*xyncols+1] += (double)((f-img)/dsize[1]) * *f;
-              xy[*plab*xyncols+2] += (double)((f-img)%dsize[1]) * *f;
+              xy[*plab*xyncols  ] += ss;
+              xy[*plab*xyncols+1] += (double)coord[0] * ss;
+              xy[*plab*xyncols+2] += (double)coord[1] * ss;
             }
         }
 
@@ -555,7 +613,7 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           plabend = (plab=worklab->array) + worklab->size;
           do
             if( *plab!=GAL_BLANK_INT32
-                && ( area[*plab]<p->detsnminarea || brightness[*plab]<0) )
+                && ( area[*plab]<p->snminarea || brightness[*plab]<0) )
               *plab=0;
           while(++plab<plabend);
         }
@@ -572,22 +630,16 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   for(i=1;i<tablen;++i)
     {
       ave=brightness[i]/area[i];
-      if( area[i]>p->detsnminarea && ave>0.0f && xy[i*xyncols]>0.0f )
+      if( area[i]>p->snminarea && ave>0.0f && xy[i*xyncols]>0.0f )
         {
           /* Get the flux weighted center coordinates. */
           coord[0]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+1]/xy[i*xyncols] );
           coord[1]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+2]/xy[i*xyncols] );
 
-          /* Calculate the Sky and Sky standard deviation on this tile. */
-          ave -= ((float *)(p->sky->array))[
-                         gal_tile_full_id_from_coord(&p->cp.tl, coord) ];
+          /* Get the Sky standard deviation on this tile. */
           err  = ((float *)(p->std->array))[
                          gal_tile_full_id_from_coord(&p->cp.tl, coord) ];
 
-          /* If the image was already sky subtracted, the second power of
-             the error needs to be doubled. */
-          err *= p->skysubtracted ? err : 2.0f*err;
-
           /* Correct the index in the sn to store the Signal to noise
              ratio. When we are dealing with the noise, we only want the
              non-zero signal to noise values, so we will just use a
@@ -597,7 +649,7 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           ind = s0d1D2 ? i : counter++;
           if(snind) indarr[ind]=i;
           snarr[ind] = ( sqrt( (float)(area[i])/p->cpscorr )
-                         * ave / sqrt(ave+err) );
+                         * ave / sqrt( ave + err*err ) );
         }
       else
         /* In detection pseudo-detections, order matters, so we will set
@@ -610,6 +662,11 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
     }
 
 
+  /* A small sanity check. */
+  if( s0d1D2==0 && counter==0 )
+    error(EXIT_FAILURE, 0, "no sky pseudo-detections.");
+
+
   /* If we are in Sky mode, the sizes have to be corrected */
   if(s0d1D2==0)
     {
@@ -648,8 +705,8 @@ detection_pseudo_remove_low_sn(struct noisechiselparams *p,
   float *snarr=sn->array;
   uint8_t *b=workbin->array;
   int32_t *l=worklab->array, *lf=l+worklab->size;
-  uint8_t *keep=gal_data_calloc_array(GAL_TYPE_UINT8, sn->size, __func__,
-                                      "keep");
+  uint8_t *keep=gal_pointer_allocate(GAL_TYPE_UINT8, sn->size, 1, __func__,
+                                     "keep");
 
   /* Specify the new labels for those that must be kept/changed. Note that
      when an object didn't have an S/N, its S/N was given a value of NaN
@@ -715,19 +772,20 @@ detection_pseudo_real(struct noisechiselparams *p)
     error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found over "
           "the sky region to estimate an S/N. This is less than %zu (value "
           "to `--minnumfalse' option). Please adjust parameters like "
-          "`--dthresh', `--detsnminarea', or make sure that there actually "
+          "`--dthresh', `--snminarea', or make sure that there actually "
           "is sufficient sky area after initial detection. You can use "
           "`--checkdetection' to see every step until this point", sn->size,
           p->minnumfalse);
 
 
   /* Get the S/N quantile and report it if we are in non-quiet mode. */
-  quant=gal_statistics_quantile(sn, p->detquant, 1);
+  quant=gal_statistics_quantile(sn, p->snquant, 1);
   p->detsnthresh = *((float *)(quant->array));
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "Pseudo-det S/N: %.2f (%.2f quant of %zu).",
-               p->detsnthresh, p->detquant, sn->size);
+      if( asprintf(&msg, "Pseudo-det S/N: %.3f (%g quant of %zu).",
+                   p->detsnthresh, p->snquant, sn->size)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
@@ -766,8 +824,9 @@ detection_final_remove_small_sn(struct noisechiselparams *p,
   gal_data_t *sn, *snind;
   int32_t *l, *lf, curlab=1;
   gal_list_str_t *comments=NULL;
-  int32_t *newlabs=gal_data_calloc_array(GAL_TYPE_INT32, num+1, __func__,
-                                         "newlabs");
+  char *extname="GROWN_DETECTION_SN";
+  int32_t *newlabs=gal_pointer_allocate(GAL_TYPE_INT32, num+1, 1, __func__,
+                                        "newlabs");
 
   /* Get the Signal to noise ratio of all detections. */
   sn=detection_sn(p, p->olabel, num, 2, "DILATED");
@@ -813,9 +872,8 @@ detection_final_remove_small_sn(struct noisechiselparams *p,
       gal_list_str_add(&comments, "See also: `DILATED' "
                        "HDU of output with `--checkdetection'.", 1);
       gal_list_str_add(&comments, "S/N of finally grown detections.", 1);
-
-
-      threshold_write_sn_table(p, sn, snind, p->detsn_D_name, comments);
+      threshold_write_sn_table(p, sn, snind, p->detsn_D_name, comments,
+                               extname);
       gal_list_str_free(comments, 1);
 
     }
@@ -854,11 +912,11 @@ detection_remove_false_initial(struct noisechiselparams *p,
 {
   size_t i;
   uint8_t *b=workbin->array;
+  float *e_th, *arr=p->conv->array;
   int32_t *l=p->olabel->array, *lf=l+p->olabel->size, curlab=1;
-  int32_t *newlabels=gal_data_calloc_array(GAL_TYPE_UINT32,
-                                           p->numinitialdets+1, __func__,
-                                           "newlabels");
-
+  int32_t *newlabels=gal_pointer_allocate(GAL_TYPE_UINT32,
+                                          p->numinitialdets+1, 1, __func__,
+                                          "newlabels");
 
   /* Find the new labels for all the existing labels. Recall that
      `newlabels' was initialized to zero, so any label that is not given a
@@ -866,7 +924,7 @@ detection_remove_false_initial(struct noisechiselparams *p,
      a label overlaps with dbyt[i], we don't need to check the rest of that
      object's pixels. At this point, tokeep is only binary: 0 or 1.
 
-     Note that the zeroth element of tokeep can also be non zero, this is
+     Note that the zeroth element of `tokeep' can also be non zero, this is
      because the holes of the labeled regions might be filled during
      filling the holes, but have not been filled in the original labeled
      array. They are not important so you can just ignore them. */
@@ -890,27 +948,59 @@ detection_remove_false_initial(struct noisechiselparams *p,
   for(i=0;i<p->numinitialdets;++i) if(newlabels[i]) newlabels[i] = curlab++;
 
 
-  /* Replace the byt and olab values with their proper values. If the user
-     doesn't want to grow, then change the labels in `lab' too. Otherwise,
-     you don't have to worry about the label array. After dilation a new
-     labeling will be done and the whole labeled array will be re-filled.*/
-  b=workbin->array; l=p->olabel->array;
-  if(p->detgrowquant==1.0f)          /* We need the binary array even when */
-    do                               /* there is no growth: the binary     */
-      {                              /* array is used for estimating the   */
-        if(*l!=GAL_BLANK_INT32)      /* Sky and its STD. */
-          *b = ( *l = newlabels[ *l ] ) > 0;
+  /* Replace the byt and olab values with their proper values. Note that we
+     need the binary array when there is no growth also. The binary array
+     is later used in estimating the sky and its STD.
+
+     See if growth is necessary or not. Note that even if the user has
+     asked for growth, if the edges of the objects in the image are sharp
+     enough, no growth will be necessary (and thus the labeled image won't
+     be re-written during growth). So it is necessary to check for growth
+     here and later do it in `detection_quantile_expand'. */
+  p->numexpand=0;
+  b=workbin->array;
+  l=p->olabel->array;
+  if(p->detgrowquant==1.0)
+    do
+      {                                       /* No growth will happen.  */
+        if(*l!=GAL_BLANK_INT32)               /* So set both the binary  */
+          *b = ( *l = newlabels[ *l ] ) > 0;  /* AND label images.       */
         ++b;
       }
     while(++l<lf);
   else
-    do
-      {
-        if(*l!=GAL_BLANK_INT32)
-          *b = newlabels[ *l ] > 0;
-        ++b;
-      }
-    while(++l<lf);
+    {
+      /* Expand the threshold values (from one value for each tile) to the
+         whole dataset. Since we know the tiles cover the whole image, we
+         don't neeed to initialize or check for blanks. */
+      p->exp_thresh_full=gal_tile_block_write_const_value(p->expand_thresh,
+                                                          p->cp.tl.tiles, 0,
+                                                          0);
+
+      /* Remove the false detections and count how many pixels need to
+         grow. */
+      e_th=p->exp_thresh_full->array;
+      do                                    /* Growth is necessary later.  */
+        {                                   /* So there is no need to set  */
+          if(*l!=GAL_BLANK_INT32)           /* the labels image, but we    */
+            {                               /* have to count the number of */
+              *b = newlabels[ *l ] > 0;     /* pixels to (possibly) grow.  */
+              if( *b==0 && *arr>*e_th )
+                ++p->numexpand;
+            }
+          ++b; ++arr; ++e_th;
+        }
+      while(++l<lf);
+
+      /* If there aren't any pixels to later expand, then reset the labels
+         (remove false detections in the labeled image). */
+      if(p->numexpand==0)
+        {
+          b=workbin->array;
+          l=p->olabel->array;
+          do if(*l!=GAL_BLANK_INT32) *l = newlabels[ *l ]; while(++l<lf);
+        }
+    }
 
 
   /* Clean up and return. */
@@ -922,92 +1012,94 @@ detection_remove_false_initial(struct noisechiselparams *p,
 
 
 
+/* Expand the initial detections based on the quantile threshold and then
+   label the connected regions. If expansion is not possible, then return
+   the `GAL_BLANK_SIZET'.*/
 static size_t
 detection_quantile_expand(struct noisechiselparams *p, gal_data_t *workbin)
 {
   int32_t *o, *of;
-  size_t *d, counter=0, numexpanded;
+  size_t *d, numexpanded=0;
+  gal_data_t *diffuseindexs;
   float *i, *e_th, *arr=p->conv->array;
-  gal_data_t *exp_thresh_full, *diffuseindexs;
   uint8_t *b=workbin->array, *bf=b+workbin->size;
 
-  /* Expand the threshold values (from one value for each tile) to the
-     whole dataset. Since we know the tiles cover the whole image, we don't
-     neeed to initialize or check for blanks.*/
-  exp_thresh_full=gal_tile_block_write_const_value(p->expand_thresh,
-                                                   p->cp.tl.tiles, 0, 0);
-
-
-  /* Count the pixels that must be expanded. */
-  e_th=exp_thresh_full->array;
-  do { if(*b++==0 && *arr>*e_th) ++counter; ++arr; ++e_th; } while(b<bf);
-
-  /* Allocate the space necessary to keep the index of all the pixels that
-     must be expanded and re-initialize the necessary pointers. */
-  diffuseindexs=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &counter, NULL, 0,
-                               p->cp.minmapsize, NULL, NULL, NULL);
-
-  /* Fill in the diffuse indexs and initialize the objects dataset. */
-  b=workbin->array;
-  arr=p->conv->array;
-  d=diffuseindexs->array;
-  e_th=exp_thresh_full->array;
-  of=(o=p->olabel->array)+p->olabel->size;
-  do
+  /* Only continue if there actually are any pixels to expand (this can
+     happen!). */
+  if(p->numexpand)
     {
-      /* If the binary value is 1, then we want an initial label of 1 (the
-         object is already detected). If it isn't, then we only want it if
-         it is above the threshold. */
-      *o = *b==1 ? 1 : ( *arr>*e_th ? CLUMPS_INIT : 0);
-      if(*b==0 && *arr>*e_th)
-        *d++ = o - (int32_t *)(p->olabel->array);
+      /* Allocate the space necessary to keep the index of all the pixels
+         that must be expanded and re-initialize the necessary pointers. */
+      diffuseindexs=gal_data_alloc(NULL, GAL_TYPE_SIZE_T, 1, &p->numexpand,
+                                   NULL, 0, p->cp.minmapsize, NULL, NULL,
+                                   NULL);
 
-      /* Increment the points and go onto the next pixel. */
-      ++b;
-      ++arr;
-      ++e_th;
-    }
-  while(++o<of);
-
-  /* Expand the detections. */
-  clumps_grow(p->olabel, diffuseindexs, 0, p->olabel->ndim);
-
-  /* Only keep the 1 valued pixels in the binary array and fill its
-     holes. */
-  o=p->olabel->array;
-  bf=(b=workbin->array)+workbin->size;
-  do *b = (*o++ == 1); while(++b<bf);
-  workbin=gal_binary_dilate(workbin, 1, 1, 1);
-  gal_binary_fill_holes(workbin, 1, p->detgrowmaxholesize);
-
-  /* Get the labeled image. */
-  numexpanded=gal_binary_connected_components(workbin, &p->olabel,
-                                              workbin->ndim);
-
-  /* Set all the input's blank pixels to blank in the labeled and binary
-     arrays. */
-  if( gal_blank_present(p->input, 1) )
-    {
-      b=workbin->array;
-      i=p->input->array;
-      of=(o=p->olabel->array)+p->olabel->size;
+      /* Fill in the diffuse indexs and initialize the objects dataset. */
+      b    = workbin->array;
+      arr  = p->conv->array;
+      d    = diffuseindexs->array;
+      e_th = p->exp_thresh_full->array;
+      of   = (o=p->olabel->array) + p->olabel->size;
       do
         {
-          if(isnan(*i++))
-            {
-              *o=GAL_BLANK_INT32;
-              *b=GAL_BLANK_UINT8;
-            }
+          /* If the binary value is 1, then we want an initial label of 1
+             (the object is already detected). If it isn't, then we only
+             want it if it is above the threshold. */
+          *o = *b==1 ? 1 : ( *arr>*e_th ? GAL_LABEL_INIT : 0);
+          if(*b==0 && *arr>*e_th)
+            *d++ = o - (int32_t *)(p->olabel->array);
+
+          /* Increment the points and go onto the next pixel. */
           ++b;
+          ++arr;
+          ++e_th;
         }
       while(++o<of);
+
+      /* Expand the detections. Note that because we are only concerned
+         with those regions that are touching a detected region, it is
+         irrelevant to sort the dataset. */
+      gal_label_grow_indexs(p->olabel, diffuseindexs, 0, p->olabel->ndim);
+
+      /* Only keep the 1 valued pixels in the binary array and fill its
+         holes. */
+      o=p->olabel->array;
+      bf=(b=workbin->array)+workbin->size;
+      do *b = (*o++ == 1); while(++b<bf);
+      workbin=gal_binary_dilate(workbin, 1, 1, 1);
+      gal_binary_holes_fill(workbin, 1, p->detgrowmaxholesize);
+
+      /* Get the labeled image. */
+      numexpanded=gal_binary_connected_components(workbin, &p->olabel,
+                                                  workbin->ndim);
+
+      /* Set all the input's blank pixels to blank in the labeled and
+         binary arrays. */
+      if( gal_blank_present(p->input, 1) )
+        {
+          b=workbin->array;
+          i=p->input->array;
+          of=(o=p->olabel->array)+p->olabel->size;
+          do
+            {
+              if(isnan(*i++))
+                {
+                  *o=GAL_BLANK_INT32;
+                  *b=GAL_BLANK_UINT8;
+                }
+              ++b;
+            }
+          while(++o<of);
+        }
+
+      /* Clean up. */
+      gal_data_free(diffuseindexs);
     }
 
   /* Clean up and return */
   gal_data_free(p->expand_thresh);
-  gal_data_free(exp_thresh_full);
-  gal_data_free(diffuseindexs);
-  return numexpanded;
+  gal_data_free(p->exp_thresh_full);
+  return numexpanded ? numexpanded : GAL_BLANK_SIZE_T;
 }
 
 
@@ -1022,7 +1114,7 @@ detection(struct noisechiselparams *p)
   char *msg;
   gal_data_t *workbin;
   struct timeval t0, t1;
-  size_t num_true_initial;
+  size_t num_true_initial, num_expanded;
 
   /* Report for the user. */
   if(!p->cp.quiet)
@@ -1044,8 +1136,9 @@ detection(struct noisechiselparams *p)
   threshold_apply(p, p->sky->array, p->std->array, THRESHOLD_SKY_STD);
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "Pseudo-detection thresh (%.3f sigma) applied.",
-               p->dthresh);
+      if( asprintf(&msg, "Pseudo-detection thresh (%.3f sigma) applied.",
+                   p->dthresh)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
@@ -1060,10 +1153,18 @@ detection(struct noisechiselparams *p)
      pseudo-detections. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
   num_true_initial=detection_remove_false_initial(p, workbin);
+  if(p->detectionname)
+    {
+      workbin->name="DETECTIONS-INIT-TRUE";
+      gal_fits_img_write(workbin, p->detectionname, NULL,
+                         PROGRAM_NAME);
+      workbin->name=NULL;
+    }
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "%zu false initial detections removed.",
-               p->numinitialdets - num_true_initial);
+      if( asprintf(&msg, "%zu false initial detections removed.",
+                   p->numinitialdets - num_true_initial)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
@@ -1073,14 +1174,31 @@ detection(struct noisechiselparams *p)
      final number of detections. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
   if(p->detgrowquant!=1.0f)
-    num_true_initial=detection_quantile_expand(p, workbin);
-  if(!p->cp.quiet)
     {
-      if(p->detgrowquant==1.0f)
-        asprintf(&msg, "%zu detections with no growth.", num_true_initial);
+      num_expanded=detection_quantile_expand(p, workbin);
+      if(num_expanded!=GAL_BLANK_SIZE_T)
+        num_true_initial=num_expanded;
+    }
+
+
+  /* Update the user on the progress, if necessary. */
+  if(!p->cp.quiet && p->detgrowquant!=1.0f && num_expanded!=GAL_BLANK_SIZE_T )
+    {
+      /* If the user hasn't asked for a labeled image, then don't confuse
+         them with the number of detections, just let them know that growth
+         is complete. */
+      if(p->label)
+        {
+          if( asprintf(&msg, "%zu detections after growth to %.3f "
+                       "quantile.", num_true_initial, p->detgrowquant)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       else
-        asprintf(&msg, "%zu detections after growth to %.3f quantile.",
-                 num_true_initial, p->detgrowquant);
+        {
+          if( asprintf(&msg, "Growth to %.3f quantile complete.",
+                       p->detgrowquant)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
@@ -1098,7 +1216,17 @@ detection(struct noisechiselparams *p)
                        : num_true_initial );
   if(!p->cp.quiet)
     {
-      asprintf(&msg, "%zu final true detections.", p->numdetections);
+      /* If the user hasn't asked for a labeled image, then don't confuse
+         them with the number of detections, just let them know that growth
+         is complete. */
+      if(p->label)
+        {
+          if( asprintf(&msg, "%zu final true detections.",
+                       p->numdetections)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+        }
+      else
+        gal_checkset_allocate_copy("Detection complete.", &msg);
       gal_timing_report(&t0, msg, 1);
       free(msg);
     }
