@@ -157,15 +157,17 @@ noisechisel_output(struct noisechiselparams *p)
     }
 
 
-  /* Write the object labels and useful information into it's header. */
+  /* Write the detected pixels and useful information into it's header. */
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "DETSN", 0, &p->detsnthresh,
                         0, "Minimum S/N of true pseudo-detections", 0,
                         "ratio");
+  if(p->label)
+    gal_fits_key_list_add(&keys, GAL_TYPE_SIZE_T, "NUMLABS", 0,
+                          &p->numdetections, 0, "Total number of labels "
+                          "(inclusive)", 0, "counter");
+  gal_fits_key_list_reverse(&keys);
   if(p->label)
     {
-      gal_fits_key_list_add(&keys, GAL_TYPE_SIZE_T, "NUMLABS", 0,
-                            &p->numdetections, 0, "Total number of labels "
-                            "(inclusive)", 0, "counter");
       p->olabel->name = "DETECTIONS";
       gal_fits_img_write(p->olabel, p->cp.output, keys, PROGRAM_NAME);
       p->olabel->name=NULL;
@@ -202,6 +204,13 @@ noisechisel_output(struct noisechiselparams *p)
                              p->cp.output, keys, PROGRAM_NAME);
   p->std->name=NULL;
 
+
+  /* Write the configuration keywords. */
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_config(&p->cp.okeys, "NoiseChisel configuration",
+                            "NOISECHISEL-CONFIG", p->cp.output, "0");
+
+
   /* Let the user know that the output is written. */
   if(!p->cp.quiet)
     printf("  - Output written to `%s'.\n", p->cp.output);
