@@ -92,9 +92,9 @@ statistics_print_one_row(struct statisticsparams *p)
 {
   int mustfree;
   char *toprint;
-  size_t dsize=1;
   double arg, *d;
   gal_list_i32_t *tmp;
+  size_t dsize=1, counter;
   gal_data_t *tmpv, *out=NULL, *num=NULL, *min=NULL, *max=NULL;
   gal_data_t *sum=NULL, *med=NULL, *meanstd=NULL, *modearr=NULL;
 
@@ -145,6 +145,7 @@ statistics_print_one_row(struct statisticsparams *p)
 
 
   /* Print every requested number. */
+  counter=0;
   for(tmp=p->singlevalue; tmp!=NULL; tmp=tmp->next)
     {
       /* By default don't free anything. */
@@ -188,12 +189,16 @@ statistics_print_one_row(struct statisticsparams *p)
           break;
         }
 
-      /* Print the number. */
+      /* Print the number. Note that we don't want any extra white space
+         characters before or after the printed outputs. So we have defined
+         `counter' to add a single white space character before any element
+         except the first one. */
       toprint=gal_type_to_string(out->array, out->type, 0);
-      printf("%s ", toprint);
+      printf("%s%s", counter ? " " : "", toprint);
       free(toprint);
 
       /* Clean up (if necessary). */
+      ++counter;
       if(mustfree) gal_data_free(out);
     }
 
@@ -255,6 +260,9 @@ statistics_interpolate_and_write(struct statisticsparams *p,
 
   /* Write the values. */
   gal_tile_full_values_write(values, &cp->tl, 1, output, NULL, PROGRAM_NAME);
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
+                            "STATISTICS-CONFIG", output, "0");
 }
 
 
@@ -588,7 +596,16 @@ write_output_table(struct statisticsparams *p, gal_data_t *table,
 
   /* Write the table. */
   gal_checkset_writable_remove(output, 0, p->cp.dontdelete);
-  gal_table_write(table, comments, p->cp.tableformat, output, "TABLE");
+  gal_table_write(table, comments, p->cp.tableformat, output, "TABLE", 0);
+
+
+  /* Write the configuration information if we have a FITS output. */
+  if(!strcmp(fix, "fits"))
+    {
+      gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+      gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
+                                "STATISTICS-CONFIG", output, "0");
+    }
 
 
   /* Let the user know, if we aren't in quiet mode. */
@@ -661,7 +678,6 @@ save_hist_and_or_cfp(struct statisticsparams *p)
   /* Set the output file name. */
   write_output_table(p, bins, suf, contents);
 
-
   /* Clean up. */
   gal_data_free(range);
 }
