@@ -3,9 +3,9 @@ Table - View and manipulate a FITS table structures.
 Table is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -46,7 +46,8 @@ struct tableparams
   struct gal_options_common_params cp; /* Common parameters.            */
   char              *filename;  /* Input filename.                      */
   gal_list_str_t     *columns;  /* List of given columns.               */
-  uint8_t         information;  /* ==1, only print FITS information.    */
+  uint8_t         information;  /* ==1: only print FITS information.    */
+  uint8_t     colinfoinstdout;  /* ==1: print column metadata in CL.    */
 
   /* Output: */
   gal_data_t           *table;  /* Linked list of output table columns. */
