@@ -3,9 +3,9 @@ Convolve -- Convolve a dataset with a given kernel.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -31,6 +31,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/list.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/convolve.h>
 #include <gnuastro/dimension.h>
 
@@ -271,8 +272,8 @@ convolve_spatial_overlap(struct per_thread_spatial_prm *pprm, int tocorrect)
   /* Set the starting point of the dataset overlap tile. */
   increment=gal_dimension_coord_to_index(ndim, block->dsize,
                                          pprm->overlap_start);
-  pprm->i_overlap->array=gal_data_ptr_increment(block->array, increment,
-                                                block->type);
+  pprm->i_overlap->array=gal_pointer_increment(block->array, increment,
+                                               block->type);
 
 
   /* Set the starting point of the kernel overlap tile. */
@@ -281,8 +282,8 @@ convolve_spatial_overlap(struct per_thread_spatial_prm *pprm, int tocorrect)
                 : gal_dimension_coord_to_index(ndim,
                                                kernel->dsize,
                                                pprm->kernel_start) );
-  pprm->k_overlap->array=gal_data_ptr_increment(kernel->array, increment,
-                                                kernel->type);
+  pprm->k_overlap->array=gal_pointer_increment(kernel->array, increment,
+                                               kernel->type);
   return full_overlap;
 }
 
@@ -423,8 +424,8 @@ convolve_spatial_on_thread(void *inparam)
   size_t i;
   size_t ndim=block->ndim;
   struct per_thread_spatial_prm *pprm=&cprm->pprm[tprm->id];
-  size_t *dsize=gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim, __func__,
-                                      "dsize");
+  size_t *dsize=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
+                                     "dsize");
 
 
   /* Set all dsize values to 1 (the values within `overlap->dsize' will be
@@ -434,15 +435,14 @@ convolve_spatial_on_thread(void *inparam)
 
   /* Initialize/Allocate necessary items for this thread. */
   pprm->cprm          = cprm;
-  pprm->pix           = gal_data_malloc_array(GAL_TYPE_SIZE_T, 2*ndim,
-                                              __func__, "pprm->pix");
-  pprm->host_start    = gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim,
-                                              __func__, "pprm->host_start");
-  pprm->kernel_start  = gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim,
-                                              __func__, "pprm->kernel_start");
-  pprm->overlap_start = gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim,
-                                               __func__,
-                                              "pprm->overlap_start");
+  pprm->pix           = gal_pointer_allocate(GAL_TYPE_SIZE_T, 2*ndim, 0,
+                                             __func__, "pprm->pix");
+  pprm->host_start    = gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0,
+                                             __func__, "pprm->host_start");
+  pprm->kernel_start  = gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0,
+                                             __func__, "pprm->kernel_start");
+  pprm->overlap_start = gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0,
+                                             __func__, "pprm->overlap_start");
   pprm->i_overlap     = gal_data_alloc(NULL, block->type, ndim, dsize,
                                        NULL, 0, -1, NULL, NULL, NULL);
   pprm->k_overlap     = gal_data_alloc(NULL, cprm->kernel->type, ndim, dsize,
@@ -501,6 +501,17 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
     error(EXIT_FAILURE, 0, "%s: only accepts `float32' type input and "
           "kernel currently", __func__);
 
+  /* It may happen that an input dataset is part of a linked list, but it
+     is not actually a tile structure (the user wants to convolve the whole
+     dataset without using tiles)! In that case, this function should break
+     beacuse a linked list is interpretted as a tile structure here.*/
+  if( tiles->block==NULL && tiles->next && tiles->next->block==NULL )
+    error(EXIT_FAILURE, 0, "%s: the input is a linked list but not a "
+          "tessellation (a list of tiles). This function is optimized to "
+          "work on a list of tiles. Please (temporarily) set the `next' "
+          "element of the input to `NULL' and call this function again",
+          __func__);
+
 
   /* Set the output datastructure.  */
   if(tocorrect) out=tocorrect;
@@ -559,6 +570,10 @@ gal_data_t *
 gal_convolve_spatial(gal_data_t *tiles, gal_data_t *kernel,
                      size_t numthreads, int edgecorrection, int convoverch)
 {
+  /* When there isn't any tile structure, `convoverch' must be set to
+     one. Recall that the input can be a single full dataset also. */
+  if(tiles->block==NULL) convoverch=1;
+
   /* Call the general function. */
   return gal_convolve_spatial_general(tiles, kernel, numthreads,
                                       edgecorrection, convoverch, NULL);
@@ -582,7 +597,7 @@ gal_convolve_spatial_correct_ch_edge(gal_data_t *tiles, gal_data_t *kernel,
   gal_data_t *block=gal_tile_block(tiles);
 
   /* Some small sanity checks. */
-  if( gal_data_dsize_is_different(block, tocorrect) )
+  if( gal_dimension_is_different(block, tocorrect) )
     error(EXIT_FAILURE, 0, "%s: the `tocorrect' dataset has to have the "
           "same dimensions/size as the block of the `tiles' input", __func__);
   if( block->type != tocorrect->type )
