@@ -3,9 +3,9 @@ data -- Structure and functions to represent/work with data
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -27,18 +27,17 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <fcntl.h>
 #include <float.h>
 #include <ctype.h>
-#include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <dirent.h>
 #include <inttypes.h>
-#include <sys/mman.h>
 
 #include <gnuastro/wcs.h>
 #include <gnuastro/data.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/table.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/checkset.h>
 
@@ -62,196 +61,31 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 /*********************************************************************/
-/*************          Size and allocation        *******************/
+/*************              Allocation             *******************/
 /*********************************************************************/
-int
-gal_data_dsize_is_different(gal_data_t *first, gal_data_t *second)
+/* Allocate a data structure based on the given parameters. If you want to
+   force the array into the hdd/ssd (mmap it), then set minmapsize=-1
+   (largest possible size_t value), in this way, no file will be larger. */
+gal_data_t *
+gal_data_alloc(void *array, uint8_t type, size_t ndim, size_t *dsize,
+               struct wcsprm *wcs, int clear, size_t minmapsize,
+               char *name, char *unit, char *comment)
 {
-  size_t i;
-
-  /* First make sure that the dimensionality is the same. */
-  if(first->ndim!=second->ndim)
-    return 1;
-
-  /* Check if the sizes along all dimensions are the same: */
-  for(i=0;i<first->ndim;++i)
-    if( first->dsize[i] != second->dsize[i] )
-      return 1;
-
-  /* If it got to here, we know the dimensions have the same length. */
-  return 0;
-}
-
-
-
-
-
-/* Increment a give pointer depending on the given type.
-
-   When working with the `array' elements of `gal_data_t', we are actually
-   dealing with `void *' pointers. Pointer arithmetic doesn't apply to
-   `void *', because the system doesn't know how much space each element
-   has to increment the pointer respectively.
-
-   So, here, we will use the type information to find the increment. This
-   is mainly useful when dealing with the `block' pointer of a tile over a
-   larger image. This function reads the address as a `char *' type (note
-   that `char' is guaranteed to have a size of 1 (byte)). It then
-   increments the `char *' by `increment*sizeof(type)' */
-void *
-gal_data_ptr_increment(void *pointer, size_t increment, uint8_t type)
-{
-  char *p=(char *)pointer;
-  return p + increment * gal_type_sizeof(type);
-}
-
-
-
-
-
-/* Find the distance between two void pointers with a given type. See the
-   explanations before `gal_data_ptr_increment'. */
-size_t
-gal_data_ptr_dist(void *earlier, void *later, uint8_t type)
-{
-  char *e=(char *)earlier, *l=(char *)later;
-  return (l-e)/gal_type_sizeof(type);
-}
-
-
-
-
-
-/* Allocate an array based on the value of type. Note that the argument
-   `size' is the number of elements, necessary in the array, the number of
-   bytes each element needs will be determined internaly by this function
-   using the datatype argument, so you don't have to worry about it. */
-void *
-gal_data_malloc_array(uint8_t type, size_t size, const char *funcname,
-                      const char *varname)
-{
-  void *array;
+  gal_data_t *out;
 
+  /* Allocate the space for the actual structure. */
   errno=0;
-  array=malloc( size * gal_type_sizeof(type) );
-  if(array==NULL)
-    {
-      if(varname)
-        error(EXIT_FAILURE, errno, "%s: %zu bytes couldn't be allocated "
-              "for variable `%s'", funcname ? funcname : __func__,
-              size * gal_type_sizeof(type), varname);
-      else
-        error(EXIT_FAILURE, errno, "%s: %zu bytes couldn't be allocated",
-              funcname ? funcname : __func__, size * gal_type_sizeof(type));
-    }
+  out=malloc(sizeof *out);
+  if(out==NULL)
+    error(EXIT_FAILURE, errno, "%s: %zu bytes for gal_data_t",
+          __func__, sizeof *out);
 
-  return array;
-}
+  /* Initialize the allocated array. */
+  gal_data_initialize(out, array, type, ndim, dsize, wcs, clear, minmapsize,
+                      name, unit, comment);
 
-
-
-
-
-void *
-gal_data_calloc_array(uint8_t type, size_t size, const char *funcname,
-                      const char *varname)
-{
-  void *array;
-
-  errno=0;
-  array=calloc( size, gal_type_sizeof(type) );
-  if(array==NULL)
-    {
-      if(varname)
-        error(EXIT_FAILURE, errno, "%s: %zu bytes couldn't be allocated "
-              "for variable `%s'", funcname ? funcname : __func__,
-              size * gal_type_sizeof(type), varname);
-      else
-        error(EXIT_FAILURE, errno, "%s: %zu bytes couldn't be allocated",
-              funcname ? funcname : __func__, size * gal_type_sizeof(type));
-    }
-
-  return array;
-}
-
-
-
-
-
-static void
-gal_data_mmap(gal_data_t *data, int clear, size_t minmapsize)
-{
-  int filedes;
-  uint8_t uc=0;
-  char *filename;
-  size_t bsize=data->size*gal_type_sizeof(data->type);
-
-
-  /* Check if the .gnuastro folder exists, write the file there. If it
-     doesn't exist, then make the .gnuastro directory.*/
-  gal_checkset_mkdir(".gnuastro");
-
-
-  /* Set the filename */
-  gal_checkset_allocate_copy("./.gnuastro/mmap_XXXXXX", &filename);
-
-
-  /* Create a zero-sized file and keep its descriptor.  */
-  errno=0;
-  /*filedes=open(filename, O_RDWR | O_CREAT | O_EXCL | O_TRUNC );*/
-  filedes=mkstemp(filename);
-  if(filedes==-1)
-    error(EXIT_FAILURE, errno, "%s: %s couldn't be created",
-          __func__, filename);
-
-
-  /* Make enough space to keep the array data. */
-  errno=0;
-  if( lseek(filedes, bsize, SEEK_SET) == -1 )
-    error(EXIT_FAILURE, errno, "%s: %s: unable to change file position by "
-          "%zu bytes", __func__, filename, bsize);
-
-
-  /* Write to the newly set file position so the space is allocated. To do
-     this, we are simply writing `uc' (a byte with value 0) into the space
-     we identified by `lseek' (above). This will ensure that this space is
-     set a side for this array and prepare us to use `mmap'. */
-  if( write(filedes, &uc, 1) == -1)
-    error(EXIT_FAILURE, errno, "%s: %s: unable to write one byte at the "
-          "%zu-th position", __func__, filename, bsize);
-
-
-  /* Map the memory. */
-  errno=0;
-  data->array=mmap(NULL, bsize, PROT_READ | PROT_WRITE, MAP_SHARED,
-                   filedes, 0);
-  if(data->array==MAP_FAILED)
-    {
-      if(minmapsize<10000u)
-        fprintf(stderr, "\nIf the processing involves many small mappings "
-                "(along with larger ones), the following error may be "
-                "corrected with a larger value to `minmapsize' (minimum "
-                "number of bytes to use mapping instead of RAM for each "
-                "patch of memory), for example 10000. In this way, mapping "
-                "will only be reserved for larger sizes. The current value is "
-                "%zu.\n\n", minmapsize);
-      error(EXIT_FAILURE, errno, "couldn't map %zu bytes into the file `%s'",
-            bsize, filename);
-    }
-
-
-  /* Close the file. */
-  if( close(filedes) == -1 )
-    error(EXIT_FAILURE, errno, "%s: %s couldn't be closed",
-          __func__, filename);
-
-
-  /* Keep the filename. */
-  data->mmapname=filename;
-
-
-  /* If it was supposed to be cleared, then clear the memory. */
-  if(clear) memset(data->array, 0, bsize);
+  /* Return the final structure. */
+  return out;
 }
 
 
@@ -357,16 +191,13 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
             {
               if( gal_type_sizeof(type)*data->size > minmapsize )
                 /* Allocate the space into disk (HDD/SSD). */
-                gal_data_mmap(data, clear, minmapsize);
+                data->array=gal_pointer_allocate_mmap(data->type, data->size,
+                                                      clear, &data->mmapname);
               else
                 /* Allocate the space in RAM. */
-                data->array = ( clear
-                                ? gal_data_calloc_array(data->type,
-                                                        data->size, __func__,
-                                                        "data->array")
-                                : gal_data_malloc_array(data->type,
-                                                        data->size, __func__,
-                                                        "data->array") );
+                data->array = gal_pointer_allocate(data->type, data->size,
+                                                   clear, __func__,
+                                                   "data->array");
             }
           else data->array=NULL; /* The given size was zero! */
         }
@@ -383,35 +214,6 @@ gal_data_initialize(gal_data_t *data, void *array, uint8_t type,
 
 
 
-/* Allocate a data structure based on the given parameters. If you want to
-   force the array into the hdd/ssd (mmap it), then set minmapsize=-1
-   (largest possible size_t value), in this way, no file will be larger. */
-gal_data_t *
-gal_data_alloc(void *array, uint8_t type, size_t ndim, size_t *dsize,
-               struct wcsprm *wcs, int clear, size_t minmapsize,
-               char *name, char *unit, char *comment)
-{
-  gal_data_t *out;
-
-  /* Allocate the space for the actual structure. */
-  errno=0;
-  out=malloc(sizeof *out);
-  if(out==NULL)
-    error(EXIT_FAILURE, errno, "%s: %zu bytes for gal_data_t",
-          __func__, sizeof *out);
-
-  /* Initialize the allocated array. */
-  gal_data_initialize(out, array, type, ndim, dsize, wcs, clear, minmapsize,
-                      name, unit, comment);
-
-  /* Return the final structure. */
-  return out;
-}
-
-
-
-
-
 /* Free the allocated contents of a data structure, not the structure
    itsself. The reason that this function is separate from `gal_data_free'
    is that the data structure might be allocated as an array (statically
@@ -450,7 +252,7 @@ gal_data_free_contents(gal_data_t *data)
   if(data->type==GAL_TYPE_STRING && data->array)
     {
       strarr=data->array;
-      for(i=0;i<data->size;++i) free(strarr[i]);
+      for(i=0;i<data->size;++i) if(strarr[i]) free(strarr[i]);
     }
 
   /* Free the array. */
@@ -533,9 +335,13 @@ gal_data_array_calloc(size_t size)
       out[i].type       = GAL_TYPE_INVALID;
       out[i].ndim       = 0;
       out[i].dsize      = NULL;
+      out[i].size       = 0;
+      out[i].mmapname   = NULL;
+      out[i].minmapsize = -1;
       out[i].nwcs       = 0;
       out[i].wcs        = NULL;
-      out[i].mmapname   = NULL;
+      out[i].flag       = 0;
+      out[i].status     = 0;
       out[i].next       = NULL;
       out[i].block      = NULL;
       out[i].name = out[i].unit = out[i].comment = NULL;
@@ -678,7 +484,11 @@ data_copy_from_string(gal_data_t *from, gal_data_t *to)
     CTYPE *a=from->array;                                               \
     for(i=0;i<from->size;++i)                                           \
       {                                                                 \
-        if(a[i]!=BLANK) asprintf(&strarr[i], FMT, a[i]);                \
+        if(a[i]!=BLANK)                                                 \
+          {                                                             \
+            if( asprintf(&strarr[i], FMT, a[i])<0 )                     \
+              error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__); \
+          }                                                             \
         else                                                            \
           gal_checkset_allocate_copy(GAL_BLANK_STRING, &strarr[i]);     \
       }                                                                 \
@@ -690,7 +500,11 @@ data_copy_from_string(gal_data_t *from, gal_data_t *to)
       {                                                                 \
         if(isnan(BLANK)) isblank = isnan(a[i]) ? 1 : 0;                 \
         else             isblank = a[i]==BLANK ? 1 : 0;                 \
-        if(isblank==0) asprintf(&strarr[i], "%f", a[i]);                \
+        if(isblank==0)                                                  \
+          {                                                             \
+            if( asprintf(&strarr[i], "%f", a[i])<0 )                    \
+              error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__); \
+          }                                                             \
         else gal_checkset_allocate_copy(GAL_BLANK_STRING, &strarr[i]);  \
       }                                                                 \
   }
@@ -963,12 +777,21 @@ gal_data_copy_to_allocated(gal_data_t *in, gal_data_t *out)
           "of dimensions, the dimensions are %zu and %zu respectively",
           __func__, out->ndim, in->ndim);
 
+  /* Free possibly allocated meta-data strings. */
+  if(out->name)    free(out->name);
+  if(out->unit)    free(out->unit);
+  if(out->comment) free(out->comment);
+
   /* Write the basic meta-data. */
   out->flag           = in->flag;
   out->next           = in->next;
   out->status         = in->status;
   out->disp_width     = in->disp_width;
   out->disp_precision = in->disp_precision;
+  gal_checkset_allocate_copy(in->name,    &out->name);
+  gal_checkset_allocate_copy(in->unit,    &out->unit);
+  gal_checkset_allocate_copy(in->comment, &out->comment);
+
 
   /* Do the copying. */
   switch(out->type)
