@@ -4,9 +4,9 @@ Common parameters between all programs.
 IMPORTANT: This header must only be included the programs, not libraries.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -60,7 +60,7 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_KEY_SEARCHIN,
       "STR",
       0,
-      "Select column(s) in: `name', `unit', `comment'.",
+      "Select column(s): `name', `unit', `comment'.",
       GAL_OPTIONS_GROUP_INPUT,
       &cp->searchin,
       GAL_TYPE_STRING,
@@ -82,6 +82,19 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "stdintimeout",
+      GAL_OPTIONS_KEY_STDINTIMEOUT,
+      "INT",
+      0,
+      "Micro-seconds to wait for standard input.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &cp->stdintimeout,
+      GAL_TYPE_LONG,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -212,7 +225,7 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_KEY_OUTPUT,
       "STR",
       0,
-      "Output name.",
+      "Output file name.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &cp->output,
       GAL_TYPE_STRING,
@@ -227,10 +240,10 @@ struct argp_option gal_commonopts_options[] =
       0,
       "Type of output: e.g., int16, float32, etc...",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &cp->type,
-      GAL_TYPE_STRING,
-      GAL_OPTIONS_RANGE_GT_0,
-      GAL_OPTIONS_NOT_MANDATORY,
+      &cp->type,                /* Internally, `cp->type' is actually an   */
+      GAL_TYPE_STRING,          /* `uint8_t', but the user gives a string. */
+      GAL_OPTIONS_RANGE_GT_0,   /* So for the sanity checks to pass, we    */
+      GAL_OPTIONS_NOT_MANDATORY,/* use `GAL_TYPE_STRING' for this option.  */
       GAL_OPTIONS_NOT_SET,
       gal_options_read_type
     },
@@ -427,7 +440,7 @@ struct argp_option gal_commonopts_options[] =
       0,
       "Only run if the program version is STR.",
       GAL_OPTIONS_GROUP_OPERATING_MODE,
-      NULL,
+      &cp->onlyversion,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
