@@ -26,6 +26,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/tile.h>
 #include <gnuastro/list.h>
+#include <gnuastro/fits.h>
 
 
 
@@ -106,7 +107,8 @@ enum options_common_keys
   GAL_OPTIONS_KEY_REMAINDERFRAC = 'F',
 
   /* Only long option (integers for keywords). */
-  GAL_OPTIONS_KEY_MINMAPSIZE   = 500,
+  GAL_OPTIONS_KEY_STDINTIMEOUT = 500,
+  GAL_OPTIONS_KEY_MINMAPSIZE,
   GAL_OPTIONS_KEY_LOG,
   GAL_OPTIONS_KEY_CITE,
   GAL_OPTIONS_KEY_CONFIG,
@@ -182,6 +184,7 @@ struct gal_options_common_params
   char                    *hdu; /* Image extension.                       */
   uint8_t             searchin; /* Column meta-data to match/search.      */
   uint8_t           ignorecase; /* Ignore case when matching col info.    */
+  long            stdintimeout; /* Timeout (micro-seconds) for stdin.     */
 
   /* Output. */
   char                 *output; /* Directory containg output.             */
@@ -203,6 +206,9 @@ struct gal_options_common_params
   uint8_t           setusrconf; /* To write teh user config config file.  */
   uint8_t           lastconfig; /* This is the last configuration file.   */
 
+  /* Output files. */
+  gal_fits_list_key_t  *okeys;  /* Configuration as FITS keys in output.  */
+
   /* For internal (to option processing) purposes. */
   uint8_t                 keep; /* Output file can exist.                 */
   void         *program_struct; /* Host program's main variable struct.   */
@@ -297,6 +303,12 @@ gal_options_set_from_key(int key, char *arg, struct argp_option *options,
 error_t
 gal_options_common_argp_parse(int key, char *arg, struct argp_state *state);
 
+char *
+gal_options_stdin_error(long stdintimeout, int precedence, char *name);
+
+gal_list_str_t *
+gal_options_check_stdin(char *inputname, long stdintimeout, char *name);
+
 
 
 
@@ -319,4 +331,7 @@ gal_options_read_config_set(struct gal_options_common_params *cp);
 void
 gal_options_print_state(struct gal_options_common_params *cp);
 
+void
+gal_options_as_fits_keywords(struct gal_options_common_params *cp);
+
 #endif
