@@ -3,9 +3,9 @@ Functions to facilitate using the Git library.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -24,7 +24,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define __GAL_GIT_H__
 
 
-#if GAL_GNUASTRO_HAVE_LIBGIT2 == 1
+#if GAL_CONFIG_HAVE_LIBGIT2 == 1
 #include <git2.h>
 #endif
 
