@@ -3,9 +3,9 @@ tile -- work with tesselations over a host dataset.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -27,6 +27,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
    must be included before the C++ preparations below */
 #include <gnuastro/data.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/dimension.h>
 
 /* C++ Preparations */
 #undef __BEGIN_C_DECLS
@@ -203,7 +204,7 @@ gal_tile_full_free_contents(struct gal_tile_two_layer_params *tl);
       {                                                                 \
         if( OTHER==tpo_oblock )    /* `OTHER' is a block. */            \
           {                                                             \
-            if(gal_data_dsize_is_different(tpo_iblock, tpo_oblock) )    \
+            if( gal_dimension_is_different(tpo_iblock, tpo_oblock) )    \
               {                                                         \
                 /* `error' function, is a GNU extension, see above. */  \
                 fprintf(stderr, "GAL_TILE_PO_OISET: when "              \
@@ -216,7 +217,7 @@ gal_tile_full_free_contents(struct gal_tile_two_layer_params *tl);
               }                                                         \
           }                                                             \
         else                                                            \
-          if(gal_data_dsize_is_different(IN, OTHER) )                   \
+          if( gal_dimension_is_different(IN, OTHER) )                   \
             {                                                           \
               /* The `error' function, is a GNU extension and this */   \
               /* is a header, not a library which the user has to  */   \
