@@ -3,9 +3,9 @@ Type -- Type information and basic operations.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -106,9 +106,11 @@ enum gal_types
 #endif
 
 #if GAL_CONFIG_SIZEOF_LONG == 4
-#define GAL_TYPE_LONG GAL_TYPE_INT32
+#define GAL_TYPE_LONG  GAL_TYPE_INT32
+#define GAL_TYPE_ULONG GAL_TYPE_UINT32
 #elif GAL_CONFIG_SIZEOF_LONG == 8
-#define GAL_TYPE_LONG GAL_TYPE_INT64
+#define GAL_TYPE_LONG  GAL_TYPE_INT64
+#define GAL_TYPE_ULONG GAL_TYPE_UINT64
 #endif
 
 
