@@ -30,6 +30,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/git.h>
 #include <gnuastro/txt.h>
+#include <gnuastro/fits.h>
 #include <gnuastro/list.h>
 #include <gnuastro/data.h>
 #include <gnuastro/table.h>
@@ -296,12 +297,16 @@ gal_options_print_citation(struct argp_option *option, char *arg,
                "Acknowledgement\n"
                "---------------\n"
                "This work was partly done using GNU Astronomy Utilities "
-               "(Gnuastro, ascl.net/1801.009) version %s. Gnuastro is a "
-               "generic package for astronomical data manipulation and "
-               "analysis which was initially created and developed for "
-               "research funded by the Monbukagakusho (Japanese "
-               "government) scholarship and European Research Council "
-               "(ERC) advanced grant 339659-MUSICOS.", PACKAGE_VERSION)<0 )
+               "(Gnuastro, ascl.net/1801.009) version %s. Work on Gnuastro "
+               "has been funded by the Japanese Ministry of Education, "
+               "Culture, Sports, Science, and Technology (MEXT) scholarship "
+               "and its Grant-in-Aid for Scientific Research (21244012, "
+               "24253003), the European Research Council (ERC) advanced "
+               "grant 339659-MUSICOS, European Union’s Horizon 2020 research "
+               "and innovation programme under Marie Sklodowska-Curie grant "
+               "agreement No 721463 to the SUNDIAL ITN, and from the Spanish "
+               "Ministry of Economy and Competitiveness (MINECO) under grant "
+               "number AYA2016-76219-P. ", PACKAGE_VERSION)<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   printf("%s\n", gnuastro_acknowledgement);
   free(gnuastro_acknowledgement);
@@ -1358,6 +1363,50 @@ gal_options_common_argp_parse(int key, char *arg, struct argp_state *state)
 
 
 
+char *
+gal_options_stdin_error(long stdintimeout, int precedence, char *name)
+{
+  char *out;
+
+  if( asprintf(&out, "no %s!\n\n"
+               "The %s can be read from a file (specified as an argument), "
+               "or the standard input.%s Standard input can come from a "
+               "pipe (output of another program) or typed on the "
+               "command-line before %ld micro-seconds (configurable with "
+               "the `--stdintimeout' option).", name, name,
+               ( precedence
+                 ? " If both are provided, a file takes precedence."
+                 : "" ), stdintimeout )<0 )
+    error(EXIT_FAILURE, 0, "%s: `asprintf' allocation error", __func__);
+
+  return out;
+}
+
+
+
+
+
+/* Make the notice that is printed before program terminates, when no input
+   is given and Standard input is also available. */
+gal_list_str_t *
+gal_options_check_stdin(char *inputname, long stdintimeout, char *name)
+{
+  gal_list_str_t *lines=inputname ? NULL : gal_txt_stdin_read(stdintimeout);
+
+  /* See if atleast one of the two inputs is given. */
+  if(inputname==NULL && lines==NULL)
+    error( EXIT_FAILURE, 0, "%s", gal_options_stdin_error(stdintimeout,
+                                                          1, name));
+
+  /* Return the output. */
+  return lines;
+}
+
+
+
+
+
+
 
 
 
@@ -2195,3 +2244,83 @@ gal_options_print_state(struct gal_options_common_params *cp)
             "and `setusrconf' options can be called in each run");
     }
 }
+
+
+
+
+
+/* Main working function for common and program specific options. */
+static void
+options_as_fits_keywords_write(gal_fits_list_key_t **keys,
+                               struct argp_option *options,
+                               struct gal_options_common_params *cp)
+{
+  size_t i;
+  void *vptr;
+  uint8_t vtype;
+  char *name, *doc;
+  gal_list_str_t *tmp;
+
+  for(i=0; !gal_options_is_last(&options[i]); ++i)
+    if( options[i].set && option_is_printable(&options[i]) )
+      {
+        /* Linked lists (multiple calls to an option). */
+        if(gal_type_is_list(options[i].type))
+          for(tmp=*(gal_list_str_t **)(options[i].value);
+              tmp!=NULL; tmp=tmp->next)
+            {
+              /* `name' and `doc' have a `const' qualifier. */
+              gal_checkset_allocate_copy(options[i].name, &name);
+              gal_checkset_allocate_copy(options[i].doc,  &doc);
+              gal_fits_key_list_add(keys, GAL_TYPE_STRING, name, 1, tmp->v,
+                                    0, doc, 1, NULL);
+            }
+        /* Normal types. */
+        else
+          {
+            /* If the option is associated with a special function for
+               reading and writing, we'll need to write the value as a
+               string. */
+            if(options[i].func)
+              {
+                vtype=GAL_TYPE_STRING;
+                vptr=options[i].func(&options[i], NULL, NULL, (size_t)(-1),
+                                      cp->program_struct);
+              }
+            else
+              {
+                vtype=options[i].type;
+                vptr = ( vtype==GAL_TYPE_STRING
+                         ? *((char **)(options[i].value))
+                         : options[i].value );
+              }
+
+            /* Write the keyword. Note that `name' and `doc' have a `const'
+               qualifier. */
+            gal_checkset_allocate_copy(options[i].name, &name);
+            if(vtype==GAL_TYPE_STRING && strlen(vptr)>FLEN_KEYWORD)
+              gal_fits_key_write_filename(name, vptr, keys, 1);
+            else
+              {
+                gal_checkset_allocate_copy(options[i].doc,  &doc);
+                gal_fits_key_list_add(keys, vtype, name, 1, vptr, 0, doc, 1,
+                                      NULL);
+              }
+          }
+      }
+}
+
+
+
+
+/* Write all options as FITS keywords. */
+void
+gal_options_as_fits_keywords(struct gal_options_common_params *cp)
+{
+  /* Write all the command and program-specific options. */
+  options_as_fits_keywords_write(&cp->okeys, cp->coptions, cp);
+  options_as_fits_keywords_write(&cp->okeys, cp->poptions, cp);
+
+  /* Reverse the list (its a first-in-first-out list). */
+  gal_fits_key_list_reverse(&cp->okeys);
+}
