@@ -3,9 +3,9 @@ Permutation -- Work on permutations (arrays of indexs).
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -26,6 +26,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <stdlib.h>
 
+#include <gnuastro/pointer.h>
 #include <gnuastro/permutation.h>
 
 
@@ -100,7 +101,7 @@ gal_permutation_apply(gal_data_t *input, size_t *permutation)
     {
       /* Necessary initializations. */
       width=gal_type_sizeof(input->type);
-      tmp=gal_data_malloc_array(input->type, 1, __func__, "tmp");
+      tmp=gal_pointer_allocate(input->type, 1, 0, __func__, "tmp");
 
       /* Do the permutation. */
       for(i=0;i<input->size;++i)
@@ -150,8 +151,8 @@ gal_permutation_apply_inverse(gal_data_t *input, size_t *permutation)
     {
       /* Initializations */
       width=gal_type_sizeof(input->type);
-      tmp=gal_data_malloc_array(input->type, 1, __func__, "tmp");
-      ttmp=gal_data_malloc_array(input->type, 1, __func__, "ttmp");
+      tmp=gal_pointer_allocate(input->type, 1, 0, __func__, "tmp");
+      ttmp=gal_pointer_allocate(input->type, 1, 0, __func__, "ttmp");
 
       /* Re-order the values. */
       for(i=0;i<input->size;++i)
