@@ -1421,10 +1421,23 @@ gal_statistics_no_blank_sorted(gal_data_t *input, int inplace)
       else
         sorted=noblank;
     }
-
   /* When the input's size is zero, just return the actual input. */
   else
-    sorted=input;
+    sorted = inplace ? input : gal_data_copy(input);
+
+  /* Set the blank and sorted flags if the dataset has zero-elements. Even
+     if having blank values or being sorted is not defined on a
+     zero-element dataset, it is up to different functions to choose what
+     they will do with a zero-element dataset. The flags have to be set
+     after this function any way. */
+  if(sorted->size==0)
+    {
+      sorted->flag |= GAL_DATA_FLAG_SORT_CH;
+      sorted->flag |= GAL_DATA_FLAG_BLANK_CH;
+      sorted->flag |= GAL_DATA_FLAG_SORTED_I;
+      sorted->flag &= ~GAL_DATA_FLAG_HASBLANK;
+      sorted->flag &= ~GAL_DATA_FLAG_SORTED_D;
+    }
 
 
   /* Return final array. */
@@ -1537,7 +1550,9 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
             {
               tmp=gal_data_copy_to_new_type_free(gal_statistics_maximum(input),
                                                  GAL_TYPE_FLOAT64);
-              max=*((double *)(tmp->array))+1e-6;
+              max=*((double *)(tmp->array));
+
+              /* Clean up. */
               gal_data_free(tmp);
             }
           else max=ra[1];
@@ -1555,7 +1570,9 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
       gal_data_free(tmp);
       tmp=gal_data_copy_to_new_type_free(gal_statistics_maximum(input),
                                          GAL_TYPE_FLOAT64);
-      max=*((double *)(tmp->array)) + 1e-6;
+      max=*((double *)(tmp->array));
+
+      /* Clean up. */
       gal_data_free(tmp);
     }
 
@@ -1592,7 +1609,7 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
   printf("onebinstart: %.10f\n", onebinstart);
   printf("binwidth: %g\n", binwidth);
   for(i=0;i<numbins;++i)
-    printf("%zu: %.4f\t(%f, %f)\n", i, b[i], b[i]-hbw, b[i]+hbw);
+    printf("%zu: %.4g\t(%g, %g)\n", i, b[i], b[i]-hbw, b[i]+hbw);
   */
 
   /* Set the status of the bins to regular and return. */
@@ -1613,7 +1630,13 @@ gal_statistics_regular_bins(gal_data_t *input, gal_data_t *inrange,
 
 #define HISTOGRAM_TYPESET(IT) {                                         \
     IT *a=input->array, *af=a+input->size;                              \
-    do if( *a>=min && *a<max) ++h[ (size_t)( (*a-min)/binwidth ) ];     \
+    do                                                                  \
+      if(*a>=min)                                                       \
+        {                                                               \
+          h_i=(*a-min)/binwidth;                                        \
+          if(h_i == last_i) { if( *a<=max ) ++h[ h_i ]; }               \
+          else              { if( *a< max ) ++h[ h_i ]; }               \
+        }                                                               \
     while(++a<af);                                                      \
   }
 
@@ -1621,9 +1644,9 @@ gal_data_t *
 gal_statistics_histogram(gal_data_t *input, gal_data_t *bins, int normalize,
                          int maxone)
 {
-  size_t *h;
   float *f, *ff;
   gal_data_t *hist;
+  size_t *h, h_i, last_i;
   double *d, min, max, ref=NAN, binwidth;
 
 
@@ -1655,8 +1678,9 @@ gal_statistics_histogram(gal_data_t *input, gal_data_t *bins, int normalize,
   /* Set the minimum and maximum range of the histogram from the bins. */
   d=bins->array;
   binwidth=d[1]-d[0];
-  max = d[bins->size - 1] + binwidth/2;
-  min = d[0]              - binwidth/2;
+  last_i=bins->size-1;
+  min = d[ 0      ] - binwidth/2;
+  max = d[ last_i ] + binwidth/2;
 
 
   /* Go through all the elements and find out which bin they belong to. */
@@ -2070,61 +2094,101 @@ gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
 
 
 
-#if 0
-/* Using the cumulative distribution function this function will
-   remove outliers from a dataset. */
-void
-gal_statistics_remove_outliers_flat_cdf(float *sorted, size_t *outsize)
+/* Find the first outlier in a distribution. */
+#define OUTLIER_BYTYPE(IT) {                                            \
+    IT *arr=nbs->array;                                                 \
+    for(i=window_size;i<nbs->size;++i)                                  \
+      {                                                                 \
+        /* Fill in the distance array. */                               \
+        for(j=0; j<wtakeone; ++j)                                       \
+          darr[j] = arr[i-window_size+j+1] - arr[i-window_size+j];      \
+                                                                        \
+        /* Get the sigma-clipped information. */                        \
+        sclip=gal_statistics_sigma_clip(dist, sigclip_multip,           \
+                                        sigclip_param, 0, 1);           \
+        sarr=sclip->array;                                              \
+                                                                        \
+        /* For a check */                                               \
+        if(quiet==0)                                                    \
+          printf("%f [%zu]: %f (%f, %f) %f\n", (float)(arr[i]), i,      \
+                 (float)(arr[i]-arr[i-1]), sarr[1], sarr[3],            \
+                 (((double)(arr[i]-arr[i-1])) - sarr[1])/sarr[3]);      \
+                                                                        \
+        /* Terminate the loop if the dist. is larger than requested. */ \
+        /* This shows we have reached the first outlier's position. */  \
+        if( (((double)(arr[i]-arr[i-1])) - sarr[1]) > sigma*sarr[3] )   \
+          {                                                             \
+            /* Allocate the output dataset. */                          \
+            out=gal_data_alloc(NULL, input->type, 1, &one, NULL, 0, -1, \
+                               NULL, NULL, NULL);                       \
+                                                                        \
+            /* Write the outlier, clean up and break. */                \
+            *(IT *)(out->array)=arr[i-1];                               \
+            gal_data_free(sclip);                                       \
+            break;                                                      \
+          }                                                             \
+                                                                        \
+        /* Clean up (if we get here). */                                \
+        gal_data_free(sclip);                                           \
+      }                                                                 \
+  }
+gal_data_t *
+gal_statistics_outlier_positive(gal_data_t *input, size_t window_size,
+                                float sigma, float sigclip_multip,
+                                float sigclip_param, int inplace, int quiet)
 {
-  printf("\n ... in gal_statistics_remove_outliers_flat_cdf ... \n");
-  exit(1);
-  int firstfound=0;
-  size_t size=*outsize, i, maxind;
-  float *slopes, minslope, maxslope;
+  float *sarr;
+  double *darr;
+  size_t i, j, one=1, wtakeone;
+  gal_data_t *dist, *sclip, *nbs, *out=NULL;
 
-  /* Find a slopes array, think of the cumulative frequency plot when
-     you want to think about slopes. */
-  errno=0; slopes=malloc(size*sizeof *slopes);
-  if(slopes==NULL)
-    error(EXIT_FAILURE, errno, "%s: %zu bytes for slopes",
-          __func__, size*sizeof *slopes);
+  /* Remove all blanks and sort the dataset. */
+  nbs=gal_statistics_no_blank_sorted(input, inplace);
 
-  /* Calcuate the slope of the CDF and put it in the slopes array. */
-  for(i=1;i<size-1;++i)
-    slopes[i]=2/(sorted[i+1]-sorted[i-1]);
+  /* Only continue if the window size is more than 2 elements (out
+     "outlier" is hard to define on smaller datasets). */
+  if(window_size>2)
+    {
+      /* For a check.
+      if(nbs->type==GAL_TYPE_FLOAT32)
+        {
+          float *n=nbs->array;
+          for(i=0;i<nbs->size;++i)
+            printf("%f\n", n[i]);
+          exit(0);
+        }
+      */
 
-  /* Find the position of the maximum slope, note that around the
-     distribution mode, the difference between the values varies less,
-     so two neighbouring elements have the closest values, hence the
-     largest slope (when their difference is in the denominator). */
-  gal_statistics_f_max_with_index(slopes+1, size-2, &maxslope, &maxind);
 
-  /* Find the minimum slope from the second element (for the first the
-     slope is not defined. NOTE; maxind is one smaller than it should
-     be because the input array to find it began from the second
-     element. */
-  gal_statistics_float_second_min(slopes+1, maxind+1, &minslope);
+      /* Allocate space to keep the distances. */
+      wtakeone=window_size-1;
+      dist=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &wtakeone, NULL,
+                          0, -1, NULL, NULL, NULL);
+      darr=dist->array;
 
-  /* Find the second place where the slope falls below `minslope`
-     after the maximum position. When found, add it with one to
-     account for error. Note that incase there are no outliers by this
-     definition, then the final size will be equal to size! */
-  for(i=maxind+1;i<size-1;++i)
-    if(slopes[i]<minslope)
-      {
-        if(firstfound)
-          break;
-        else
-          firstfound=1;
-      }
-  *outsize=i+1;
+      /* Find the outlier based on the type of the input dataset. */
+      switch(input->type)
+        {
+        case GAL_TYPE_UINT8:     OUTLIER_BYTYPE( uint8_t  );   break;
+        case GAL_TYPE_INT8:      OUTLIER_BYTYPE( int8_t   );   break;
+        case GAL_TYPE_UINT16:    OUTLIER_BYTYPE( uint16_t );   break;
+        case GAL_TYPE_INT16:     OUTLIER_BYTYPE( int16_t  );   break;
+        case GAL_TYPE_UINT32:    OUTLIER_BYTYPE( uint32_t );   break;
+        case GAL_TYPE_INT32:     OUTLIER_BYTYPE( int32_t  );   break;
+        case GAL_TYPE_UINT64:    OUTLIER_BYTYPE( uint64_t );   break;
+        case GAL_TYPE_INT64:     OUTLIER_BYTYPE( int64_t  );   break;
+        case GAL_TYPE_FLOAT32:   OUTLIER_BYTYPE( float    );   break;
+        case GAL_TYPE_FLOAT64:   OUTLIER_BYTYPE( double   );   break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: type code %d not recognized",
+                __func__, input->type);
+        }
 
-  /*
-  for(i=0;i<size;++i)
-    printf("%zu\t%.3f\t%.3f\n", i, arr[i], slopes[i]);
-  printf("\n\nPlace to cut off for outliers is: %zu\n\n", *outsize);
-  */
+      /* Clean up. */
+      gal_data_free(dist);
+    }
 
-  free(slopes);
+  /* Clean up and return. */
+  if(nbs!=input) gal_data_free(nbs);
+  return out;
 }
-#endif
