@@ -3,9 +3,9 @@ table -- Functions for I/O on tables.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016, Free Software Foundation, Inc.
+Copyright (C) 2016-2018, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -60,16 +60,16 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
    comments), nothing in the data structure(s) will be allocated by this
    function for the actual data (e.g., the `array' or `dsize' elements). */
 gal_data_t *
-gal_table_info(char *filename, char *hdu, size_t *numcols, size_t *numrows,
-               int *tableformat)
+gal_table_info(char *filename, char *hdu, gal_list_str_t *lines,
+               size_t *numcols, size_t *numrows, int *tableformat)
 {
   /* Get the table format and size (number of columns and rows). */
-  if(gal_fits_name_is_fits(filename))
+  if(filename && gal_fits_name_is_fits(filename))
     return gal_fits_tab_info(filename, hdu, numcols, numrows, tableformat);
   else
     {
       *tableformat=GAL_TABLE_FORMAT_TXT;
-      return gal_txt_table_info(filename, numcols, numrows);
+      return gal_txt_table_info(filename, lines, numcols, numrows);
     }
 
   /* Abort with an error if we get to this point. */
@@ -358,8 +358,10 @@ make_list_of_indexs(gal_list_str_t *cols, gal_data_t *allcols,
            are done (and program is aborted) before this step. */
         if(nummatch==0)
           {
-            asprintf(&errorstring, "`%s' didn't match any of the column %ss.",
-                     tmp->v, gal_tableintern_searchin_as_string(searchin));
+            if( asprintf(&errorstring, "`%s' didn't match any of the "
+                         "column %ss.", tmp->v,
+                         gal_tableintern_searchin_as_string(searchin))<0 )
+              error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
             gal_tableintern_error_col_selection(filename, hdu, errorstring);
           }
 
@@ -401,9 +403,9 @@ make_list_of_indexs(gal_list_str_t *cols, gal_data_t *allcols,
    So the first requested column is the first popped data structure and so
    on. */
 gal_data_t *
-gal_table_read(char *filename, char *hdu, gal_list_str_t *cols,
-               int searchin, int ignorecase, int minmapsize,
-               size_t *colmatch)
+gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
+               gal_list_str_t *cols, int searchin, int ignorecase,
+               size_t minmapsize, size_t *colmatch)
 {
   int tableformat;
   gal_list_sizet_t *indexll;
@@ -411,7 +413,8 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *cols,
   gal_data_t *allcols, *out=NULL;
 
   /* First get the information of all the columns. */
-  allcols=gal_table_info(filename, hdu, &numcols, &numrows, &tableformat);
+  allcols=gal_table_info(filename, hdu, lines, &numcols, &numrows,
+                         &tableformat);
 
   /* If there was no actual data in the file, then return NULL. */
   if(allcols==NULL) return NULL;
@@ -433,7 +436,7 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *cols,
   switch(tableformat)
     {
     case GAL_TABLE_FORMAT_TXT:
-      out=gal_txt_table_read(filename, numrows, allcols, indexll,
+      out=gal_txt_table_read(filename, lines, numrows, allcols, indexll,
                              minmapsize);
       break;
 
@@ -491,20 +494,18 @@ gal_table_comments_add_intro(gal_list_str_t **comments, char *program_string,
 
   /* Get the Git description in the running folder. */
   tmp=gal_git_describe();
-  if(tmp) sprintf(gitdescribe, " from %s,", tmp);
-  else    gitdescribe[0]='\0';
-  free(tmp);
-
+  if(tmp) { sprintf(gitdescribe, " from %s,", tmp); free(tmp); }
+  else      gitdescribe[0]='\0';
 
   /* Git version and time of program's starting, this will be the second
      line. Note that ctime puts a `\n' at the end of its string, so we'll
      have to remove that. Also, note that since we are allocating `msg', we
      are setting the allocate flag of `gal_list_str_add' to 0. */
-  asprintf(&tmp, "Created%s on %s", gitdescribe, ctime(rawtime));
+  if( asprintf(&tmp, "Created%s on %s", gitdescribe, ctime(rawtime))<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   tmp[ strlen(tmp)-1 ]='\0';
   gal_list_str_add(comments, tmp, 0);
 
-
   /* Program name: this will be the top of the list (first line). We will
      need to set the allocation flag for this one, because program_string
      is usually statically allocated.*/
@@ -521,7 +522,8 @@ gal_table_comments_add_intro(gal_list_str_t **comments, char *program_string,
    specified by `tableformat'. */
 void
 gal_table_write(gal_data_t *cols, gal_list_str_t *comments,
-                int tableformat, char *filename, char *extname)
+                int tableformat, char *filename, char *extname,
+                uint8_t colinfoinstdout)
 {
   /* If a filename was given, then the tableformat is relevant and must be
      used. When the filename is empty, a text table must be printed on the
@@ -531,11 +533,11 @@ gal_table_write(gal_data_t *cols, gal_list_str_t *comments,
       if(gal_fits_name_is_fits(filename))
         gal_fits_tab_write(cols, comments, tableformat, filename, extname);
       else
-        gal_txt_write(cols, comments, filename);
+        gal_txt_write(cols, comments, filename, colinfoinstdout);
     }
   else
     /* Write to standard output. */
-    gal_txt_write(cols, comments, filename);
+    gal_txt_write(cols, comments, filename, colinfoinstdout);
 }
 
 
@@ -553,12 +555,13 @@ gal_table_write_log(gal_data_t *logll, char *program_string,
   gal_table_comments_add_intro(&comments, program_string, rawtime);
 
   /* Write the log file to disk */
-  gal_table_write(logll, comments, GAL_TABLE_FORMAT_TXT, filename, "LOG");
+  gal_table_write(logll, comments, GAL_TABLE_FORMAT_TXT, filename, "LOG", 0);
 
   /* In verbose mode, print the information. */
   if(!quiet)
     {
-      asprintf(&msg, "%s created.", filename);
+      if( asprintf(&msg, "%s created.", filename)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(NULL, msg, 1);
       free(msg);
     }
