@@ -50,4 +50,9 @@ if [ ! -f $psf      ]; then echo "$psf does not exist.";   exit 77; fi
 
 # Actual test script
 # ==================
-$execname $img --kernel=$psf --domain=spatial --output=convolve_spatial.fits
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img --kernel=$psf --domain=spatial   \
+                              --output=convolve_spatial.fits
