@@ -49,5 +49,10 @@ if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
 
 # Actual test script
 # ==================
-$execname $img $cat --mode=img --zeroisnotblank --output=imgpolygon.fits      \
-          --polygon=209,50:436.76,151:475.64,438.2:210.6,454.04:121.4,289.88
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
+$check_with_program $execname $img $cat --mode=img --zeroisnotblank \
+                              --output=imgpolygon.fits              \
+                              --polygon=209,50:436.76,151:475.64,438.2:210.6,454.04:121.4,289.88
