@@ -4,7 +4,7 @@
 # (in the Installing gnuastro section).
 #
 # Original author:
-#     Mohammad Akhlaghi <akhlaghi@gnu.org>
+#     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 # Contributing author(s):
 #
 # Copying and distribution of this file, with or without modification,
@@ -18,7 +18,7 @@
 # Preliminaries
 # =============
 #
-# Set the variabels (The executable is in the build tree). Do the
+# Set the variables (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 #
@@ -30,7 +30,7 @@
 prog=mknoise
 img1=convolve_spatial.fits
 execname=../bin/$prog/ast$prog
-img2=convolve_spatial_warped.fits
+img2=convolve_spatial_scaled.fits
 
 
 
@@ -46,7 +46,9 @@ img2=convolve_spatial_warped.fits
 #
 #   - The input data was not made (for example the test that created the
 #     data file failed).
-if [ ! -f $execname ] || [ ! -f $img1 ] || [ ! -f $img2 ]; then exit 77; fi
+if [ ! -f $execname ]; then echo "$execname doesn't exist."; exit 77; fi
+if [ ! -f $img1     ]; then echo "$img1 does not exist.";    exit 77; fi
+if [ ! -f $img2     ]; then echo "$img2 does not exist.";    exit 77; fi
 
 
 
@@ -54,7 +56,11 @@ if [ ! -f $execname ] || [ ! -f $img1 ] || [ ! -f $img2 ]; then exit 77; fi
 
 # Actual test script
 # ==================
+#
+# `check_with_program' can be something like `Valgrind' or an empty
+# string. Such programs will execute the command if present and help in
+# debugging when the developer doesn't have access to the user's system.
 export GSL_RNG_SEED=1
 export GSL_RNG_TYPE=ranlxs2
-$execname --envseed $img1
-$execname --envseed $img2
+$check_with_program $execname --envseed $img1
+$check_with_program $execname --envseed $img2
